--------------------------------------------------------------------------------
-- patcher.lua                                                                --
-- 2013-10-24  - Pyriel                                                       --
--                                                                            --
--This script does most of the work of extracting, patching, and rebuilding   --
--the disc.                                                                   --
--                                                                            --
--------------------------------------------------------------------------------
SearchPatches =
{
	["circlet"] = { Active = 0, Type = "CIRCLET" }
  , ["gozz"] = { Active = 0, Type = "GOZU" }
  , ["godspeed"] = { Active = 0, Type = "GODSPEED" }
}

PackageInfo = {
	Name 		= "Gensou Suikoden II Bug Fix Patch",
	ShortName	= "GS2 Bug Fix Patch",
	Version		= "2.02.078",
	Date		= "2019-01-03",
	Games		= {
		{ Name = "Suikoden II", Region = "North America", Language = "English", Serial = "SLUS_009.58", PatchDir="patch\\NA",
		  Patches = {
			{
				Name = "Armor Attribute Fixes",
				Description = "Adjusts armor attributes and strips/restores resistances that were applied to incorrect items.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "armor_fx_1.ppf", GameFileName = "BP0_FST.BIN" },
					{ Type = "PPF", PatchFileName = "armor_fx_2.ppf", GameFileName = "BP0_SEC.BIN" }
				}
			},
			
			{
				Name = "Badeaux Appears",
				Description = "Badeaux will appear in the castle grounds near the pond behind the library.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "badeaux.ppf", GameFileName = "VK11.BIN" }
				}
			},

			{
				Name = "Castle Armory Fix",
				Description = "Prevents the Castle Armory from selling items when the player has insufficient potch.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "castle_armory.ppf", GameFileName = "HDOUGUYA.BIN" }
				}
			},

			{
				Name = "Castle Farm Fix",
				Description = "Fixes the Castle Farm module so the player can hand in multiple seeds or animals at one time.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "castle_farm.ppf", GameFileName = "VK19.BIN" }
				}
			},

			{
				Name="Chaco Bug Fix",
				Description="Adds a missing Set Level command to Chaco's recruit script.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "chaco.ppf", GameFileName = "VH01.BIN" }
				}
			},

			{
				Name="Circlet Name Correction",
				Description="Changes \"circuret\" to \"circlet\".",
				Active = 1, Toggle = 0,
				Search = {
					{ Value="CIRCURET", Replace="CIRCLET", Case="false", VarQual="circlet" }
				}
			},

			{
				Name="Collectibles Fix",
				Description="Repairs errors in the after-battle module that can prevent Recipes, Plans, Window Sets and Sound Sets from dropping.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "collectibles.ppf", GameFileName = "BP0_AFT.BIN" }
				}
			},

			{
				Name="Emilia Dialogue Fix",
				Description="Force Emilia to use the Hero's alias during Greenhill events.  Mostly for consistency, as the aliases are abandoned quickly by everyone else.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "emiliaspy.ppf", GameFileName = "VE07.BIN" }
				}
			},

			{
				Name="Forgiver Sign Fix",
				Description="Fix display.  Forgiver Sign does zero damage, but a huge value is displayed.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "forgiver_sign_1.ppf", GameFileName = "BP0_FST.BIN" },
					{ Type = "PPF", PatchFileName = "forgiver_sign_2.ppf", GameFileName = "BP0_SEC.BIN" }
				}
			},

			{
				Name="Godspeed Rune Name Correction",
				Description="Changes \"True Holy\" to \"Godspeed\".",
				Active = 1, Toggle = 0,
				Search = {
					{ Value="True Holy", Replace="Godspeed", Case="false", VarQual="godspeed" }
				}
			},

			{
				Name="Gozz Rune Renamed Minotaur Rune",
				Description="Changes \"Gozz\" to \"Minotaur\".",
				Active = 1, Toggle = 0,
				Search = {
					{ Value="GOZZ", Replace="MINOTAUR", Case="false", VarQual="gozz" }
				},
				Excludes = { "Gozz Rune Renamed Gozu Rune" }
			},

			{
				Name="Gozz Rune Renamed Gozu Rune",
				Description="Changes \"Gozz\" to \"Gozu\".",
				Active = 0,
				Search = {
					{ Value="GOZZ", Replace="GOZU", Case="false", VarQual="gozz" }
				},
				Excludes = { "Gozz Rune Renamed Minotaur Rune" }
			},

			{
				Name="Gregminster Rune Shop Fix",
				Description="Alters a script for the rune shop in Gregminster so only one set of rare finds is available, regardless of where the hero stands.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "gregminsterruneshop.ppf", GameFileName = "VF03.BIN" }
				}
			},

			{
				Name="Gremio Text Fix",
				Description="Fixes missing pointers for Gremio dialogues that should appear when you try to leave Banner.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "gremiotext.ppf", GameFileName = "VF05.BIN" }
				}
			},


			{
				Name="GS1 Load - Names Fix",
				Description="Fixes the transfer of the hero and castle Names from the original Genso Suikoden.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "gs1names.ppf", GameFileName = "G1LOAD.BIN" }
				}
			},

			{
				Name="GS1 Load - Tai Ho Import",
				Description="Vincent (unplayable in GS1) is removed from the import and import bonuses are added for Tai Ho",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "gs1taiho.ppf", GameFileName = "G1LOAD.BIN" }
				}
			},

			{
				Name="GS1 Load - Rune Import",
				Description="Patches the broken Rune Import so that characters with unlocked, open slots can transfer runes.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "gs1runes.ppf", GameFileName = "G1LOAD.BIN" }
				}
			},

			{
				Name="Inns Fix",
				Description="Prevent the Inn from healing the party for free.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "inn_1.ppf", GameFileName = "YADOYA1.BIN" },
					{ Type = "PPF", PatchFileName = "inn_2.ppf", GameFileName = "YADOYA2.BIN" }
				}
			},

			{
				Name="Join At 99 Fix",
				Description="Prevents recruits from joining at level 99 in error.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "join_at_99.ppf", GameFileName = "SLUS_009.58" }
				}
			},

			{
				Name="Kindness Rune Fix (No Penalty)",
				Description="When a character's invisible Kindness Rating goes below zero the Rune gives no bonus to attack instead of maxing it out.",
				Active = 0,
				Files = {
					{ Type="PPF", PatchFileName = "kindness_rune_np.ppf", GameFileName = "SLUS_009.58" }
				},
				Excludes = { "Kindness Rune Fix (Penalty)" }
			},

			{
				Name="Kindness Rune Fix (Penalty)",
				Description="When a character's invisible Kindness Rating goes below zero the Rune gives a penalty to attack instead of maxing it out.",
				Active=1,
				Files = {
					{ Type="PPF", PatchFileName = "kindness_rune.ppf", GameFileName = "SLUS_009.58" }
				},
				Excludes = { "Kindness Rune Fix (No Penalty)" }
			},

			{
				Name="Knife-thrower Fix",
				Description="Adds a custom routine for the Knife-throwing event in Ryube that keeps HP from being zeroed.",
				Active=1,
				Files = {
					{ Type = "PPF", PatchFileName = "knife_thrower.ppf", GameFileName = "VB18.BIN" }
				}
			},

			{
				Name="Lamb Fix (Unicorn Woods)",
				Description="Separates the acquired flag for the lamb from the flag for the first chest (can now get both).",
				Active=1,
				Files = {
					{ Type= "PPF", PatchFileName = "lamb.ppf", GameFileName = "VH10.BIN" }
				}
			},

			{
				Name="Luca Battle Party-change Fix",
				Description="The party is emptied but not reformed prior to fighting Luca, allowing the hero to be alone in the back row and effectively invincible.  Fix forces the party to reform.",
				Active=1,
				Files = {
					{ Type = "PPF", PatchFileName = "luca.ppf", GameFileName = "PARTYCE1.BIN" }
				}
			},

			{
				Name="Matilda Gate Fix",
				Description="Makes the Matilda Gate immovable.",
				Active=1,
				Files = {
					{ Type = "PPF", PatchFileName = "matilda_gate.ppf", GameFileName = "VI11.BIN" }
				}
			},

			{
				Name="Music Fixes",
				Description="Fix various issues with music and audio.",
				Active=1,
				Files = {
					{ Type = "PPF", PatchFileName = "bgm.ppf", GameFileName = "BGM.XA" }
				}
			},

			{
				Name="North Sparrow Escape Talisman Fix",
				Description="Prevents using an Escape Talisman in North Sparrow from allowing you to pass the guards, and return to Highland early.",
				Active=1,
				Files = {
					{ Type = "PPF", PatchFileName = "nsparrowentry.ppf", GameFileName = "VB17.BIN" }
				}
			},

			{
				Name="Rope Climb - Turtle Plans #4",
				Description="Prevents Turtle Plans #4 from being awarded multiple times by the Rope Climb mini-game.",
				Active=1,
				Files = {
					{ Type = "PPF", PatchFileName = "ropeturtleplans.ppf", GameFileName = "KIKORI.BIN" }
				}
			},

			{
				Name="Rune Speed Fix",
				Description="Fixes an intermittent bug that gives healing spells' temporary speed boost to Rune Unites.",
				Active=1,
				Files = {
					{ Type= "PPF", PatchFileName = "rune_speed_1.ppf", GameFileName = "BP0_FST.BIN" },
					{ Type= "PPF", PatchFileName = "rune_speed_2.ppf", GameFileName = "BP0_SEC.BIN" }
				}
			},

			{
				Name="Rune Unite Fix",
				Description="Repairs a bug that could allow a single character to participate in multiple Rune Unite spells.",
				Active=1,
				Files = {
					{ Type= "PPF", PatchFileName = "rune_unite_1.ppf", GameFileName = "BP0_FST.BIN" },
					{ Type= "PPF", PatchFileName = "rune_unite_2.ppf", GameFileName = "BP0_SEC.BIN" }
				}
			},

			{
				Name="Scroll Shop Display Fix",
				Description="Stops the Scroll Shop from displaying the actual Names of unidentified items.",
				Active=1,
				Files = {
					{ Type="PPF", PatchFileName = "scroll_shop.ppf", GameFileName = "FUDAZUK.BIN" }
				}
			},

			{
				Name="Tenzan Pass Enemies Fix",
				Description="Translates the Names of the enemies in Tenzan pass, and adds the formations that contain the Chimera-type monster to the encounter table.",
				Active=1,
				Files = {
					{ Type="PPF", PatchFileName = "tenzan.ppf", GameFileName = "VA11.BIN" }
				}
			},

			{
				Name="Tinto Glitch Fix",
				Description="Fixes a script error that could make it impossible to continue the game during events in Tinto",
				Active=1,
				Files = {
					{ Type="PPF", PatchFileName = "tintoscript.ppf", GameFileName = "VG01.BIN" }
				}
			},

			{
				Name="Trade Gossip Translation",
				Description="Translates the text of the trade gossip, which otherwise appears as a garbage in-game.",
				Active=1,
				Files = {
					{ Type="PPF", PatchFileName = "uwasa_1.ppf", GameFileName = "UWASA1.BIN" },
					{ Type="PPF", PatchFileName = "uwasa_2.ppf", GameFileName = "UWASA2.BIN" }
				}
			}
		  }, -- end patches

		  LBAPatchList= {
			 { name="SLUS_009.58",     path="/",     startsector=0,   seekpos=0x7dbbc },
			 { name="VJ24.BIN",     path="/CDROM/100_ARJ/",     startsector=0,   seekpos=0x23f24 },
			 { name="FURO.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x16de0 },
			 { name="HSOUND.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x3cd8 },
			 { name="RBATTLE.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x430a4 },
			 { name="SNDTEST.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x280a4 },
			 { name="IPRG_A3.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11504 },
			 { name="IPRG_A4.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11f54 },
			 { name="IPRG_B2.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x1088c },
			 { name="IPRG_D.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10e20 },
			 { name="IPRG_D2.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10e44 },
			 { name="IPRG_D3.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11364 },
			 { name="IPRG_J.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10ce4 },
			 { name="BOOT.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x250f4 },
			 { name="ENDING.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x3c40 },
			 { name="OVER.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0xbb2c },
			 { name="STAFF.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x3058 }
		  }, -- end LBAPatchList

		  FileList = {
			 { name="SYSTEM.CNF",     path="/",     lba=0,   size=0 },
			 { name="SLUS_009.58",     path="/",     lba=0,   size=0 },
			 { name="BGM.XA",     path="/CDROM/",     lba=0,   size=0 },
			 { name="A_1.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="A_2.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="BGM.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="BGM2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="CYUTON.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="CYUUTON.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="KYARO.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="KYARO1.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="ROUYA.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="ROUYA.TKD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="SEQ.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="SEQ2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_B.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_C.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_D.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_E.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_F.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_G.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_H.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA01.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA02.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA03.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA04.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA05.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA06.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA07.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA08.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA09.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA10.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA11.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA12.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA13.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA14.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA15.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA16.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA17.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA18.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA19.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA20.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA21.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA22.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA23.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA24.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA25.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA26.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="WA.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="YAKUSO1.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="YAKUSO2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="YAKUSO21.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="YAKUSO3.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="B_1.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="B_3.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="DANGION.TKD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="ENHOKU1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="ENHOK_CB.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="IKKI.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="KAISOU4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="RYUBE1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="RYUBE2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="RYUBE3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="RYUBE4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="SD_B01.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="TOTO1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="TOTO2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="TOTO3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="TOTO4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="TOTO5.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB01.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB02.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB03.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB04.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB05.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB06.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB07.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB08.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB09.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB10.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB11.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB12.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB13.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB14.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB15.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB16.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB17.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB18.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB19.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB20.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB21.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK5.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK6.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK7.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="WB.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="YOUHEI.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="YUUWAKU.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="C_1.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="C_4.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="C_5.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="FCEV_C.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="HAKUSHI1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="HAKUSHI2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="HAKUSHI3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="HAKUSHI4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="KOKKYO.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="KORONE.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_10.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_11.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_5.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_6.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_7.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_8.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_81.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_9.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE2_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE2_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE2_3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE2_4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE2_5.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE3_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE3_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="SD_C05.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC01.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC02.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC03.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC04.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC05.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC06.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC07.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC08.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC09.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC10.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC11.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC12.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC13.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC14.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC15.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC16.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC17.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC18.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC19.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC20.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC21.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC22.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC23.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC24.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC25.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC26.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC27.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC28.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC29.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC30.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC31.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="WC.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="D_1.VAB",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="KAZE1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="KAZE2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="KUSU1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="KUSUKUSU.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH4.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH5.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH6.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RADATO1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RADATO2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RADATO3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RUKAMOR1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RUKAMOR2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RUKAMOR3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="SD_D04.VAB",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="SOUTH1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="SOUTH2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="SOUTH3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD01.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD02.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD03.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD04.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD05.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD06.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD07.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD08.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD09.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD10.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD11.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD12.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD13.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD14.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD15.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD16.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD17.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD18.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD19.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD20.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="WD.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="E_1.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="E_2.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="E_3.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="E_4.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="E_5.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="E_6.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_1.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_2.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_3.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_4.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_5.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_6.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_7.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_8.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_9.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_1.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_2.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_3.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_4.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_5.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_6.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_7.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="MORIMURA.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="SEKISYO.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE01.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE02.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE03.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE04.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE05.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE06.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE07.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE08.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE09.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE10.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE11.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE12.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE13.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE14.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE15.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE16.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE17.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="WE.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="BANAR.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="BANAR2.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="ROKKAKU.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="TORAN1.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="TORAN2.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF01.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF02.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF03.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF04.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF05.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF06.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF07.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="WF.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="G_1.VAB",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KITADO.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KITADOUK.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KOKOU.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KUROMU1.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KUROMU2.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KUROMU3.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KUROMU4.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KUROMU5.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="RYUUKO.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="SD_G11.VAB",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="SOUTOU.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO1.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO2.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO21.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO3.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO4.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO5.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO6.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO7.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO8.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG01.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG02.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG03.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG04.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG05.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG06.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG07.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG08.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG09.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG10.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG11.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG12.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG13.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG14.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG15.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG16.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG17.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG18.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="WG.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="H_1.VAB",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="KOBORUT.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="LAKEWST1.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV1.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV2.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV21.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV3.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV4.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV5.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH01.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH02.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH03.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH04.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH05.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH06.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH07.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH08.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH09.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH10.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH11.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH12.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH13.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH14.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH15.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH16.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH17.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="WH.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="I_1.VAB",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="I_3.VAB",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIMURA.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU1.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU2.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU3.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU4.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU5.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU6.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU7.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU8.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU9.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="RAKUTEI.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="RAKUTEI2.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI01.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI02.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI03.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI04.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI05.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI06.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI07.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI08.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI09.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI10.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI11.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI12.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI13.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI14.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI15.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="WI.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="ANSATU1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="ANSATU2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="HILAND1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="HILAND2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="HILAND3.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="HIMURA.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="JOUI1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="J_2.VAB",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="J_3.VAB",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="LASTBOSS.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO3.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO4.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO5.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO6.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO7.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="TAIKETU1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ01.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ02.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ03.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ04.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ05.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ06.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ07.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ08.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ09.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ10.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ11.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ12.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ13.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ14.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ15.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ16.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ17.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ18.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ19.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ20.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ21.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ22.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ23.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ24.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="WAHEI1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="WJ.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="HONKYO1.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO10.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO2.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO3.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO4.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO5.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO6.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO7.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO8.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO9.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI1.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI2.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI3.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI4.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI5.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI6.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="RYOURI.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_K01.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_K29.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG1.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG2.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG3.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG4.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG5.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG6.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG7.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK01.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK02.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK03.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK04.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK05.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK06.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK07.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK08.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK09.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK10.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK11.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK12.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK13.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK14.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK15.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK16.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK17.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK18.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK19.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK20.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK21.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK22.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK23.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK24.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK25.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK26.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK27.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK28.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK29.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK30.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK31.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK32.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK33.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK34.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK35.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK36.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK37.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="WK.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="ADA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="AIR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="AIR_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ALX_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="AMD.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="AMD_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ANI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="APP_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="BIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="BOR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="BOZ.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="CRY.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="CYK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="CYK_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ERI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FIC_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FRE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FRE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FRI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FRI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FRK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FUT.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GAB.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GEO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GJM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GRE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GRI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="HAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="HAW.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="HIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="HNF.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="JIJ.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="JO2.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="JOI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KAL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KAN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KIB_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KKL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KKS.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KLK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KMU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KNI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KRU_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KSD.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KSM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KSM_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KYU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KYU_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="LFI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="LKK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MCR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MCR_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MDO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MFI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MI2.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MIL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MRL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MSM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MTF.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MTO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MTS.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MTT.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MUK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="NAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="NAN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="NIA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ORA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="PEC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="PR2_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="PRK_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="RIK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="RKL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ROL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="RYE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="RYN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SEE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SEE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SHE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SHU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SIE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SIE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SIN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SIN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SIR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SMB.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SNA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SRO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SSK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="STA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="STK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="TAI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="TAI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="TEL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="TEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="TOU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="UNI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="VAL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="VAL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="VAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="VIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="VIC_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="WIG.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="WOL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ZAI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ZAI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ZAM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ZKO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ZSH.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="DOUGUYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="DOUGUYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KAJIYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KAJIYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KANTEIY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KANTEIY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KOUEKI1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KOUEKI2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KPARTY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KPARTY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="MONSYOY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="MONSYOY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="PARTYIN1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="PARTYIN2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="SAVE1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="SAVE2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="UWASA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="UWASA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="YADOYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="YADOYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="BOOK.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="CHIE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="CHIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="CHIN2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="FISHING.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="FUDAZUK.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="FURO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="F_DAT1.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="F_DAT2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="F_DAT3.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="F_DAT4.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="F_DAT5.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HDOUGUYA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HKAJIYA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HMAP.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HMAP2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HMONSYOY.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HSOUND.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HWINDOW.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="KIKORI.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="MEYASU.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="MOGURA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="ODORIKO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="PARTYCE1.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="PARTYCH2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="PARTYCHG.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="PEEPING.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RBATTLE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA00.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA01.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA02.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA03.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA04.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA05.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA06.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA07.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA08.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA09.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA10.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="REST.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="SHUGO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="SNDTEST.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="SOUKO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="STONE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="SYUGOITM.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="TANTEI.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="TEAMCHG.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="TEAMCHG2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="TELEPO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="VB19PIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="VG08PIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="WAKEUP.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="WAKEUP2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="BP0_AFT.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			 { name="BP0_BEF.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			 { name="BP0_FST.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			 { name="BP0_MAK.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			 { name="BP0_SEC.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			 { name="BF_DAT.BIN",     path="/CDROM/160_BFDA/",     lba=0,   size=0 },
			 { name="UD0_UN01.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN02.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN03.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN04.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN05.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN06.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN07.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN08.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN09.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN10.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN11.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN12.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN13.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN14.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN15.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN16.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN17.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN18.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN19.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN20.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN21.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN22.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN23.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN24.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN25.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN26.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN27.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN28.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN29.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN30.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN31.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN32.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN33.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN34.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN35.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN36.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN37.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN38.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN39.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN40.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN41.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN42.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN43.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN44.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN45.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN46.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN47.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN48.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN49.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN50.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN51.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="MAGIC.BAK",     path="/CDROM/180_MAGI/",     lba=0,   size=0 },
			 { name="MAGIC.VAB",     path="/CDROM/180_MAGI/",     lba=0,   size=0 },
			 { name="MD0_BLN1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_BLN2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_BLN3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_BOR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_BUR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_EAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_EAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_EAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_EAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_EAT5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FIR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FIR2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FIR3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FIR4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FIR5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FUR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GRI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GRI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GRI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_HOL1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_HOL2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_HOL3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_HOL4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KAE.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KAL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KKS.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KLK1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KLK2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KLK3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KSM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_LFI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_MDO.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_MFI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_MIL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_MSM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_ORA.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_RAI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SEN.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SHI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SHI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SHI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SHI4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SIN.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP10.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP7.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP8.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP9.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SUI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SWO1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SWO2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SWO3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SWO4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SYO.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_THU1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_THU2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_THU3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_THU4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_THU5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_UNI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_UNI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_UNI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_VAL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WAT5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WIN1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WIN2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WIN3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WIN4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WIN5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WINR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WOL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_ZAM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC10.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC11.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC12.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC13.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC14.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC15.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC16.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC17.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC18.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC19.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC20.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC7.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC8.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC9.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MF_BOA0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_CIM0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_CIM1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_CIM2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_COC0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_COC1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_COL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_COL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE6.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE7.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE8.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE9.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DREA.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DREB.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DREC.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRED.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_EAR4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_FIR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_FIR3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_FIR4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_FIR5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GAR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GAR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GEK0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GEK1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GOL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GOL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GOR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GOR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GWO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GWO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_HAP0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_HAP1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_IMO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_IMO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_IMO2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_KI20.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_KI21.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_KUL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_KUL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS6.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS7.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LUS0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAG0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAG1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAG2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAG3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MEI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MEI1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_NEC0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_NEC1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_NED0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_NED1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_RAU0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_RAU1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SAM0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SEI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SEI1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SID0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SIE0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SIE1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SIR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_TAR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_TAR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_UNI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_WIN1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_WIN3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="METEO.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			 { name="SWORD.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			 { name="TATE.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			 { name="THUNDER.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			 { name="DOR.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="DOR_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="GOL.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="GOL_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="GOR.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="GWO1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="GWO2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="HAP1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="HAP2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="HAP_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="IMO1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="IMO2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="IMO3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="KI1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="KUL_SID.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LAS.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUC1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUC2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUC3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUS1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUS2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUS3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUS4.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUS5.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="NEC1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="NEC2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="NEC3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="NIK.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="NIK_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RAT.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RAT_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RAU1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RAU2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RYO.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RYO_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="SEI.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="SIE1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="EFFECT.BIN",     path="/CDROM/220_EFFE/",     lba=0,   size=0 },
			 { name="EB_VA05.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VB02.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VB02B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VB02C.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VB07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VB19.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC07B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC11.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC16.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC16B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC16C.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VD07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VD07B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE02.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE03.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE06.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE08.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE15.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VG01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VG03.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VG04.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VG07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VH01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VH08.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VJ04.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EVEFFECT.BIN",     path="/CDROM/240_EVEN/",     lba=0,   size=0 },
			 { name="FE_DAT00.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT01.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT02.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT03.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT04.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT05.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT06.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT07.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT08.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT09.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT10.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT11.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT12.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT13.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT14.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT15.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT16.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT17.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT18.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT19.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT20.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="IDAT_A3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_A4.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_B2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_D.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_D2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_D3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_J.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_A3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_A4.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_B2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_D.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_D2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_D3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_J.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="BLOAD.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="BOOT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="BOOTDAT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ENDING.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ENDING1.SDD",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ENDING2.SDD",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT4.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT5.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT6.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDA1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDA2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDA3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDA4.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDA5.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDAT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="G1LOAD.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="KNMLOGO.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="OVER.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="STAFF.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ST_END1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ST_END2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ST_END3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ALL.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="ARMS.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="ARMSD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="ARMST.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="BOGU.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="CKD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="DISH.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="DOGU.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="EMBL.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="EX_ITEM.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="FNAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="FNO.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="FOOD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="GAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="KEVENT.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="KOTEI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="MAGI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="MS.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="NAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="PARTY.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="SUKI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="TABLE.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="TOWN.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="TRADE.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="VARIA.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="WAR_DAT.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="SD_WAR.BAK",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="SD_WAR.VAB",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_01.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_02.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_03.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_04.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_05.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_06.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_07.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_08.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_09.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_10.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_11.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_12.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_13.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_14.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_15.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_16.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="BOSS0.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="BOSS1.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="BOSS2.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="DEMO1.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="DEMO2.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="DEMO3.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="DEMO5.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="DEMO6.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="KONA5OLD.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="KONAMIC5.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="LOOP.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="OP_US.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="ROCK.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="_KONAMIC.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="ZZZ.STR",     path="/",     lba=0,   size=0 }
		  } -- end file list
		},  -- end game NA


		{ Name = "Suikoden II", Region = "Europe", Language = "English", Serial = "SLES_024.42", PatchDir="patch\\European-E",
			Patches = {
			{
				Name = "Badeaux Appears",
				Description = "Badeaux will appear in the castle grounds near the pond behind the library.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "badeaux.ppf", GameFileName = "VK11.BIN" }
				}
			},

			{
				Name = "Castle Armory Fix",
				Description = "Prevents the Castle Armory from selling items when the player has insufficient potch.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "castle_armory.ppf", GameFileName = "HDOUGUYA.BIN" }
				}
			},

			{
				Name = "Castle Farm Fix",
				Description = "Fixes the Castle Farm module so the player can hand in multiple seeds or animals at one time.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "castle_farm.ppf", GameFileName = "VK19.BIN" }
				}
			},

			{
				Name="Chaco Bug Fix",
				Description="Adds a missing Set Level command to Chaco's recruit script.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "chaco.ppf", GameFileName = "VH01.BIN" }
				}
			},

			{
				Name="Circlet Name Correction",
				Description="Changes \"circuret\" to \"circlet\".",
				Active = 1, Toggle = 0,
				Search = {
					{ Value="CIRCURET", Replace="CIRCLET", Case="false", VarQual="circlet" }
				}
			},

			{
				Name="Collectibles Fix",
				Description="Repairs errors in the after-battle module that can prevent Recipes, Plans, Window Sets and Sound Sets from dropping.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "collectibles.ppf", GameFileName = "BP0_AFT.BIN" }
				}
			},

			{
				Name="Forgiver Sign Fix",
				Description="Fix display.  Forgiver Sign does zero damage, but a huge value is displayed.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "forgiver_sign_1.ppf", GameFileName = "BP0_FST.BIN" },
					{ Type = "PPF", PatchFileName = "forgiver_sign_2.ppf", GameFileName = "BP0_SEC.BIN" }
				}
			},

			{
				Name="Godspeed Rune Name Correction",
				Description="Changes \"True Holy\" to \"Godspeed\".",
				Active = 1, Toggle = 0,
				Search = {
					{ Value="True Holy", Replace="Godspeed", Case="false", VarQual="godspeed" }
				}
			},

			{
				Name="Gozz Rune Renamed Minotaur Rune",
				Description="Changes \"Gozz\" to \"Minotaur\".",
				Active = 1, Toggle = 0,
				Search = {
					{ Value="GOZZ", Replace="MINOTAUR", Case="false", VarQual="gozz" }
				},
				Excludes = { "Gozz Rune Renamed Gozu Rune" }
			},

			{
				Name="Gozz Rune Renamed Gozu Rune",
				Description="Changes \"Gozz\" to \"Gozu\".",
				Active = 0,
				Search = {
					{ Value="GOZZ", Replace="GOZU", Case="false", VarQual="gozz" }
				},
				Excludes = { "Gozz Rune Renamed Minotaur Rune" }
			},

			{
				Name="Inns Fix",
				Description="Prevent the Inn from healing the party for free.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "inn_1.ppf", GameFileName = "YADOYA1.BIN" },
					{ Type = "PPF", PatchFileName = "inn_2.ppf", GameFileName = "YADOYA2.BIN" }
				}
			},

			{
				Name="Join At 99 Fix",
				Description="Prevents recruits from joining at level 99 in error.",
				Active = 1, Toggle = 0,
				Files = {
					{ Type = "PPF", PatchFileName = "join_at_99.ppf", GameFileName = "SLES_024.42" }
				}
			},

			{
				Name="Kindness Rune Fix (No Penalty)",
				Description="When a character's invisible Kindness Rating goes below zero the Rune gives no bonus to attack instead of maxing it out.",
				Active = 0,
				Files = {
					{ Type="PPF", PatchFileName = "kindness_rune_np.ppf", GameFileName = "SLES_024.42" }
				},
				Excludes = { "Kindness Rune Fix (Penalty)" }
			},

			{
				Name="Kindness Rune Fix (Penalty)",
				Description="When a character's invisible Kindness Rating goes below zero the Rune gives a penalty to attack instead of maxing it out.",
				Active=1,
				Files = {
					{ Type="PPF", PatchFileName = "kindness_rune.ppf", GameFileName = "SLES_024.42" }
				},
				Excludes = { "Kindness Rune Fix (No Penalty)" }
			},

			{
				Name="Knife-thrower Fix",
				Description="Adds a custom routine for the Knife-throwing event in Ryube that keeps HP from being zeroed.",
				Active=1,
				Files = {
					{ Type = "PPF", PatchFileName = "knife_thrower.ppf", GameFileName = "VB18.BIN" }
				}
			},

			{
				Name="Lamb Fix (Unicorn Woods)",
				Description="Separates the acquired flag for the lamb from the flag for the first chest (can now get both).",
				Active=1,
				Files = {
					{ Type= "PPF", PatchFileName = "lamb.ppf", GameFileName = "VH10.BIN" }
				}
			},

			{
				Name="Luca Battle Party-change Fix",
				Description="The party is emptied but not reformed prior to fighting Luca, allowing the hero to be alone in the back row and effectively invincible.  Fix forces the party to reform.",
				Active=1,
				Files = {
					{ Type = "PPF", PatchFileName = "luca.ppf", GameFileName = "PARTYCE1.BIN" }
				}
			},

			{
				Name="Matilda Gate Fix",
				Description="Makes the Matilda Gate immovable.",
				Active=1,
				Files = {
					{ Type = "PPF", PatchFileName = "matilda_gate.ppf", GameFileName = "VI11.BIN" }
				}
			},

			{
				Name="Rune Unite Fix",
				Description="Repairs a bug that could allow a single character to participate in multiple Rune Unite spells.",
				Active=1,
				Files = {
					{ Type= "PPF", PatchFileName = "rune_unite_1.ppf", GameFileName = "BP0_FST.BIN" },
					{ Type= "PPF", PatchFileName = "rune_unite_2.ppf", GameFileName = "BP0_SEC.BIN" }
				}
			},

			{
				Name="Scroll Shop Display Fix",
				Description="Stops the Scroll Shop from displaying the actual Names of unidentified items.",
				Active=1,
				Files = {
					{ Type="PPF", PatchFileName = "scroll_shop.ppf", GameFileName = "FUDAZUK.BIN" }
				}
			},

			{
				Name="Tenzan Pass Enemies Fix",
				Description="Translates the Names of the enemies in Tenzan pass, and adds the formations that contain the Chimera-type monster to the encounter table.",
				Active=1,
				Files = {
					{ Type="PPF", PatchFileName = "tenzan.ppf", GameFileName = "VA11.BIN" }
				}
			}
		  },  -- end patches

		  LBAPatchList = {
			 { name="SLES_024.42",     path="/",     startsector=0,   seekpos=0x7dca0 },
			 { name="VJ24.BIN",     path="/CDROM/100_ARJ/",     startsector=0,   seekpos=0x23f24 },
			 { name="FURO.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x16dec },
			 { name="HSOUND.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x3cd8 },
			 { name="RBATTLE.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x430b0 },
			 { name="SNDTEST.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x280f0 },
			 { name="IPRG_A3.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11504 },
			 { name="IPRG_A4.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11f54 },
			 { name="IPRG_B2.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x1088c },
			 { name="IPRG_D.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10e20 },
			 { name="IPRG_D2.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10e44 },
			 { name="IPRG_D3.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11364 },
			 { name="IPRG_J.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10ce4 },
			 { name="BOOT.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x250f4 },
			 { name="ENDING.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x3c54 },
			 { name="OVER.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0xbb38 },
			 { name="STAFF.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x3060 }
		  }, -- end LBAPatchList

		  FileList = {
			 { name="SYSTEM.CNF",     path="/",     lba=0,   size=0 },
			 { name="SLES_024.42",     path="/",     lba=0,   size=0 },
			 { name="BGM.XA",     path="/CDROM/",     lba=0,   size=0 },
			 { name="A_1.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="A_2.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="BGM.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="BGM2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="CYUTON.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="CYUUTON.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="KYARO.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="KYARO1.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="ROUYA.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="ROUYA.TKD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="SEQ.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="SEQ2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_B.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_C.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_D.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_E.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_F.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_G.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="STAY_H.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA01.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA02.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA03.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA04.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA05.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA06.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA07.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA08.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA09.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA10.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA11.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA12.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA13.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA14.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA15.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA16.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA17.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA18.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA19.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA20.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA21.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA22.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA23.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA24.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA25.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="VA26.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="WA.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="YAKUSO1.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="YAKUSO2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="YAKUSO21.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="YAKUSO3.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			 { name="B_1.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="B_3.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="DANGION.TKD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="ENHOKU1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="ENHOK_CB.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="IKKI.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="KAISOU4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="RYUBE1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="RYUBE2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="RYUBE3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="RYUBE4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="SD_B01.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="TOTO1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="TOTO2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="TOTO3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="TOTO4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="TOTO5.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB01.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB02.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB03.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB04.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB05.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB06.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB07.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB08.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB09.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB10.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB11.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB12.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB13.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB14.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB15.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB16.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB17.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB18.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB19.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB20.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VB21.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK5.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK6.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="VIK7.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="WB.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="YOUHEI.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="YUUWAKU.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			 { name="C_1.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="C_4.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="C_5.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="FCEV_C.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="HAKUSHI1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="HAKUSHI2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="HAKUSHI3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="HAKUSHI4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="KOKKYO.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="KORONE.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_10.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_11.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_5.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_6.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_7.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_8.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_81.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE1_9.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE2_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE2_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE2_3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE2_4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE2_5.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE3_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="MUSE3_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="SD_C05.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC01.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC02.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC03.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC04.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC05.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC06.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC07.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC08.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC09.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC10.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC11.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC12.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC13.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC14.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC15.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC16.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC17.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC18.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC19.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC20.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC21.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC22.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC23.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC24.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC25.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC26.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC27.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC28.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC29.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC30.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="VC31.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="WC.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			 { name="D_1.VAB",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="KAZE1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="KAZE2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="KUSU1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="KUSUKUSU.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH4.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH5.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="NORTH6.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RADATO1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RADATO2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RADATO3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RUKAMOR1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RUKAMOR2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="RUKAMOR3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="SD_D04.VAB",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="SOUTH1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="SOUTH2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="SOUTH3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD01.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD02.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD03.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD04.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD05.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD06.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD07.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD08.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD09.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD10.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD11.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD12.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD13.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD14.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD15.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD16.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD17.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD18.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD19.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="VD20.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="WD.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			 { name="E_1.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="E_2.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="E_3.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="E_4.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="E_5.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="E_6.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_1.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_2.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_3.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_4.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_5.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_6.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_7.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_8.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN1_9.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_1.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_2.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_3.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_4.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_5.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_6.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="GREEN2_7.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="MORIMURA.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="SEKISYO.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE01.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE02.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE03.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE04.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE05.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE06.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE07.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE08.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE09.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE10.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE11.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE12.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE13.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE14.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE15.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE16.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="VE17.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="WE.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			 { name="BANAR.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="BANAR2.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="ROKKAKU.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="TORAN1.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="TORAN2.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF01.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF02.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF03.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF04.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF05.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF06.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="VF07.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="WF.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			 { name="G_1.VAB",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KITADO.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KITADOUK.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KOKOU.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KUROMU1.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KUROMU2.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KUROMU3.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KUROMU4.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="KUROMU5.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="RYUUKO.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="SD_G11.VAB",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="SOUTOU.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO1.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO2.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO21.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO3.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO4.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO5.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO6.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO7.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="TINTO8.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG01.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG02.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG03.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG04.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG05.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG06.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG07.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG08.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG09.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG10.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG11.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG12.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG13.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG14.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG15.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG16.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG17.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="VG18.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="WG.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			 { name="H_1.VAB",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="KOBORUT.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="LAKEWST1.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV1.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV2.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV21.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV3.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV4.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="TWORIV5.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH01.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH02.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH03.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH04.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH05.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH06.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH07.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH08.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH09.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH10.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH11.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH12.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH13.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH14.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH15.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH16.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="VH17.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="WH.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			 { name="I_1.VAB",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="I_3.VAB",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIMURA.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU1.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU2.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU3.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU4.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU5.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU6.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU7.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU8.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="MATIRU9.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="RAKUTEI.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="RAKUTEI2.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI01.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI02.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI03.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI04.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI05.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI06.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI07.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI08.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI09.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI10.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI11.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI12.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI13.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI14.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="VI15.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="WI.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			 { name="ANSATU1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="ANSATU2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="HILAND1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="HILAND2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="HILAND3.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="HIMURA.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="JOUI1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="J_2.VAB",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="J_3.VAB",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="LASTBOSS.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO3.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO4.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO5.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO6.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="RURUNO7.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="TAIKETU1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ01.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ02.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ03.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ04.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ05.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ06.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ07.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ08.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ09.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ10.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ11.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ12.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ13.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ14.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ15.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ16.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ17.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ18.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ19.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ20.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ21.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ22.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ23.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="VJ24.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="WAHEI1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="WJ.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			 { name="HONKYO1.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO10.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO2.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO3.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO4.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO5.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO6.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO7.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO8.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="HONKYO9.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI1.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI2.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI3.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI4.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI5.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="MINI6.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="RYOURI.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_K01.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_K29.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG1.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG2.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG3.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG4.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG5.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG6.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="SD_MG7.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK01.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK02.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK03.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK04.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK05.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK06.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK07.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK08.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK09.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK10.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK11.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK12.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK13.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK14.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK15.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK16.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK17.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK18.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK19.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK20.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK21.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK22.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK23.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK24.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK25.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK26.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK27.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK28.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK29.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK30.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK31.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK32.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK33.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK34.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK35.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK36.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="VK37.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="WK.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			 { name="ADA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="AIR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="AIR_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ALX_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="AMD.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="AMD_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ANI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="APP_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="BIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="BOR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="BOZ.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="CRY.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="CYK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="CYK_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ERI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FIC_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FRE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FRE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FRI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FRI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FRK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="FUT.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GAB.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GEO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GJM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GRE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="GRI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="HAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="HAW.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="HIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="HNF.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="JIJ.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="JO2.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="JOI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KAL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KAN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KIB_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KKL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KKS.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KLK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KMU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KNI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KRU_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KSD.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KSM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KSM_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KYU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="KYU_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="LFI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="LKK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MCR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MCR_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MDO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MFI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MI2.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MIL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MRL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MSM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MTF.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MTO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MTS.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MTT.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="MUK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="NAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="NAN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="NIA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ORA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="PEC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="PR2_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="PRK_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="RIK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="RKL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ROL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="RYE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="RYN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SEE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SEE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SHE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SHU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SIE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SIE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SIN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SIN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SIR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SMB.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SNA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SRO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="SSK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="STA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="STK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="TAI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="TAI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="TEL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="TEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="TOU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="UNI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="VAL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="VAL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="VAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="VIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="VIC_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="WIG.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="WOL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ZAI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ZAI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ZAM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ZKO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="ZSH.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			 { name="DOUGUYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="DOUGUYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KAJIYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KAJIYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KANTEIY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KANTEIY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KOUEKI1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KOUEKI2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KPARTY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="KPARTY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="MONSYOY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="MONSYOY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="PARTYIN1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="PARTYIN2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="SAVE1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="SAVE2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="UWASA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="UWASA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="YADOYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="YADOYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			 { name="BOOK.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="CHIE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="CHIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="CHIN2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="FISHING.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="FUDAZUK.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="FURO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="F_DAT1.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="F_DAT2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="F_DAT3.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="F_DAT4.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="F_DAT5.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HDOUGUYA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HKAJIYA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HMAP.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HMAP2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HMONSYOY.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HSOUND.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="HWINDOW.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="KIKORI.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="MEYASU.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="MOGURA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="ODORIKO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="PARTYCE1.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="PARTYCH2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="PARTYCHG.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="PEEPING.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RBATTLE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA00.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA01.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA02.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA03.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA04.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA05.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA06.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA07.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA08.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA09.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="RB_CHA10.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="REST.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="SHUGO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="SNDTEST.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="SOUKO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="STONE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="SYUGOITM.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="TANTEI.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="TEAMCHG.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="TEAMCHG2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="TELEPO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="VB19PIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="VG08PIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="WAKEUP.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="WAKEUP2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			 { name="BP0_AFT.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			 { name="BP0_BEF.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			 { name="BP0_FST.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			 { name="BP0_MAK.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			 { name="BP0_SEC.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			 { name="BF_DAT.BIN",     path="/CDROM/160_BFDA/",     lba=0,   size=0 },
			 { name="UD0_UN01.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN02.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN03.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN04.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN05.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN06.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN07.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN08.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN09.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN10.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN11.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN12.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN13.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN14.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN15.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN16.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN17.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN18.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN19.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN20.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN21.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN22.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN23.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN24.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN25.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN26.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN27.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN28.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN29.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN30.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN31.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN32.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN33.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN34.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN35.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN36.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN37.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN38.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN39.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN40.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN41.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN42.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN43.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN44.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN45.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN46.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN47.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN48.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN49.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN50.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="UD0_UN51.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			 { name="MAGIC.BAK",     path="/CDROM/180_MAGI/",     lba=0,   size=0 },
			 { name="MAGIC.VAB",     path="/CDROM/180_MAGI/",     lba=0,   size=0 },
			 { name="MD0_BLN1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_BLN2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_BLN3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_BOR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_BUR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_DAR6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_EAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_EAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_EAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_EAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_EAT5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FIR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FIR2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FIR3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FIR4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FIR5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_FUR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GRI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GRI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_GRI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_HOL1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_HOL2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_HOL3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_HOL4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KAE.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KAL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KKS.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KLK1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KLK2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KLK3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_KSM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_LFI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_MDO.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_MFI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_MIL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_MSM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_ORA.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_RAI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SEN.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SHI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SHI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SHI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SHI4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SIN.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP10.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP7.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP8.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SP9.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SUI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SWO1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SWO2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SWO3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SWO4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_SYO.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_THU1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_THU2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_THU3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_THU4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_THU5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_UNI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_UNI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_UNI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_VAL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WAT5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WIN1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WIN2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WIN3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WIN4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WIN5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WINR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_WOL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD0_ZAM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC10.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC11.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC12.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC13.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC14.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC15.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC16.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC17.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC18.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC19.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC20.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC7.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC8.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MD_SPC9.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			 { name="MF_BOA0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_CIM0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_CIM1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_CIM2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_COC0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_COC1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_COL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_COL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE6.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE7.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE8.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRE9.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DREA.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DREB.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DREC.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_DRED.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_EAR4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_FIR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_FIR3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_FIR4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_FIR5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GAR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GAR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GEK0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GEK1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GOL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GOL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GOR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GOR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GWO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_GWO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_HAP0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_HAP1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_IMO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_IMO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_IMO2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_KI20.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_KI21.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_KUL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_KUL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS6.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LAS7.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_LUS0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAG0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAG1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAG2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAG3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MAO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MEI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_MEI1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_NEC0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_NEC1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_NED0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_NED1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_RAU0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_RAU1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SAM0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SEI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SEI1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SID0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SIE0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SIE1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_SIR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_TAR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_TAR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_UNI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_WIN1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="MF_WIN3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			 { name="METEO.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			 { name="SWORD.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			 { name="TATE.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			 { name="THUNDER.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			 { name="DOR.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="DOR_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="GOL.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="GOL_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="GOR.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="GWO1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="GWO2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="HAP1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="HAP2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="HAP_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="IMO1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="IMO2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="IMO3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="KI1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="KUL_SID.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LAS.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUC1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUC2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUC3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUS1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUS2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUS3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUS4.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="LUS5.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="NEC1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="NEC2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="NEC3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="NIK.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="NIK_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RAT.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RAT_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RAU1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RAU2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RYO.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="RYO_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="SEI.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="SIE1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			 { name="EFFECT.BIN",     path="/CDROM/220_EFFE/",     lba=0,   size=0 },
			 { name="EB_VA05.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VB02.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VB02B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VB02C.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VB07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VB19.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC07B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC11.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC16.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC16B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VC16C.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VD07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VD07B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE02.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE03.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE06.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE08.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VE15.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VG01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VG03.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VG04.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VG07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VH01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VH08.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EB_VJ04.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			 { name="EVEFFECT.BIN",     path="/CDROM/240_EVEN/",     lba=0,   size=0 },
			 { name="FE_DAT00.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT01.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT02.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT03.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT04.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT05.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT06.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT07.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT08.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT09.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT10.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT11.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT12.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT13.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT14.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT15.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT16.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT17.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT18.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT19.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="FE_DAT20.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			 { name="IDAT_A3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_A4.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_B2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_D.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_D2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_D3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IDAT_J.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_A3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_A4.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_B2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_D.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_D2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_D3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="IPRG_J.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			 { name="BLOAD.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="BOOT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="BOOTDAT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ENDING.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ENDING1.SDD",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ENDING2.SDD",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT4.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT5.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="END_DAT6.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDA1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDA2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDA3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDA4.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDA5.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="EVENTDAT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="G1LOAD.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="KNMLOGO.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="OVER.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="STAFF.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ST_END1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ST_END2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ST_END3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			 { name="ALL.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="ARMS.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="ARMSD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="ARMST.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="BOGU.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="CKD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="DISH.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="DOGU.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="EMBL.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="EX_ITEM.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="FNAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="FNO.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="FOOD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="GAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="KEVENT.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="KOTEI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="MAGI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="MS.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="NAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="PARTY.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="SUKI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="TABLE.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="TOWN.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="TRADE.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="VARIA.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="WAR_DAT.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			 { name="SD_WAR.BAK",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="SD_WAR.VAB",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_01.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_02.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_03.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_04.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_05.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_06.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_07.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_08.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_09.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_10.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_11.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_12.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_13.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_14.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_15.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="WAR_16.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			 { name="BOSS0.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="BOSS1.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="BOSS2.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="DEMO1.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="DEMO2.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="DEMO3.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="DEMO5.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="DEMO6.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="KONA5OLD.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="KONAMIC5.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="LOOP.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="OP_US.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="ROCK.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="_KONAMIC.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			 { name="ZZZ.STR",     path="/",     lba=0,   size=0 }
		  } -- end filelist
		}, -- end game UK

		{ Name = "Suikoden II", Region = "Europe", Language = "German", Serial = "SLES_024.44", PatchDir="patch\\European-G",
			Patches = {
			{
				Name = "Badeaux Appears",
				Description = "Badeaux will appear in the castle grounds near the pond behind the library.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "badeaux.ppf", GameFileName = "VK11.BIN" }}
			},

			{
				Name = "Castle Armory Fix",
				Description = "Prevents the Castle Armory from selling items when the player has insufficient potch.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "castle_armory.ppf", GameFileName = "HDOUGUYA.BIN" }}
			},

			{
				Name = "Castle Farm Fix",
				Description = "Fixes the Castle Farm module so the player can hand in multiple seeds or animals at one time.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "castle_farm.ppf", GameFileName = "VK19.BIN" }}
			},

			{
				Name="Chaco Bug Fix",
				Description="Adds a missing Set Level command to Chaco's recruit script.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "chaco.ppf", GameFileName = "VH01.BIN" }}
			},

			{
				Name="Circlet Name Correction",
				Description="Changes \"circuret\" to \"circlet\".",
				Active = 1, Toggle = 0,
				Search = {{ Value="CIRCURET", Replace="CIRCLET", Case="false", VarQual="circlet" }}
			},

			{
				Name="Collectibles Fix",
				Description="Repairs errors in the after-battle module that can prevent Recipes, Plans, Window Sets and Sound Sets from dropping.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "collectibles.ppf", GameFileName = "BP0_AFT.BIN" }}
			},

			{
				Name="Forgiver Sign Fix",
				Description="Fix display.  Forgiver Sign does zero damage, but a huge value is displayed.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "forgiver_sign_1.ppf", GameFileName = "BP0_FST.BIN" },
						{ Type = "PPF", PatchFileName = "forgiver_sign_2.ppf", GameFileName = "BP0_SEC.BIN" }}
			},

			{
				Name="Godspeed Rune Name Correction",
				Description="Changes \"True Holy\" to \"Godspeed\".",
				Active = 1, Toggle = 0,
				Search = {{ Value="True Holy", Replace="Godspeed", Case="false", VarQual="godspeed" }}
			},

			{
				Name="Gozz Rune Renamed Minotaur Rune",
				Description="Changes \"Gozz\" to \"Minotaur\".",
				Active = 1, Toggle = 0,
				Search = {{ Value="GOZZ", Replace="MINOTAUR", Case="false", VarQual="gozz" }},
				Excludes = { "Gozz Rune Renamed Gozu Rune" }
			},

			{
				Name="Gozz Rune Renamed Gozu Rune",
				Description="Changes \"Gozz\" to \"Gozu\".",
				Active = 0,
				Search = {{ Value="GOZZ", Replace="GOZU", Case="false", VarQual="gozz" }},
				Excludes = { "Gozz Rune Renamed Minotaur Rune" }
			},
			
			{
				Name="Gregminster Recruiting Crash Fix",
				Description="Fixes a crash that occurs when meeting Lepant, if you take a moment to review the recruits he offers.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "gregminstercrash.ppf", GameFileName = "VF04.BIN" }}
			},

			{
				Name="Inns Fix",
				Description="Prevent the Inn from healing the party for free.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "inn_1.ppf", GameFileName = "YADOYA1.BIN" },
						{ Type = "PPF", PatchFileName = "inn_2.ppf", GameFileName = "YADOYA2.BIN" }}
			},

			{
				Name="Join At 99 Fix",
				Description="Prevents recruits from joining at level 99 in error.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "join_at_99.ppf", GameFileName = "SLES_024.44" }}
			},

			{
				Name="Kindness Rune Fix (No Penalty)",
				Description="When a character's invisible Kindness Rating goes below zero the Rune gives no bonus to attack instead of maxing it out.",
				Active = 0,
				Files = {{ Type="PPF", PatchFileName = "kindness_rune_np.ppf", GameFileName = "SLES_024.44" }},
				Excludes = { "Kindness Rune Fix (Penalty)" }
			},

			{
				Name="Kindness Rune Fix (Penalty)",
				Description="When a character's invisible Kindness Rating goes below zero the Rune gives a penalty to attack instead of maxing it out.",
				Active=1,
				Files = {{ Type="PPF", PatchFileName = "kindness_rune.ppf", GameFileName = "SLES_024.44" }},
				Excludes = { "Kindness Rune Fix (No Penalty)" }
			},

			{
				Name="Knife-thrower Fix",
				Description="Adds a custom routine for the Knife-throwing event in Ryube that keeps HP from being zeroed.",
				Active=1,
				Files = {{ Type = "PPF", PatchFileName = "knife_thrower.ppf", GameFileName = "VB18.BIN" }}
			},

			{
				Name="Lamb Fix (Unicorn Woods)",
				Description="Separates the acquired flag for the lamb from the flag for the first chest (can now get both).",
				Active=1,
				Files = {{ Type= "PPF", PatchFileName = "lamb.ppf", GameFileName = "VH10.BIN" }}
			},

			{
				Name="Luca Battle Party-change Fix",
				Description="The party is emptied but not reformed prior to fighting Luca, allowing the hero to be alone in the back row and effectively invincible.  Fix forces the party to reform.",
				Active=1,
				Files = {{ Type = "PPF", PatchFileName = "luca.ppf", GameFileName = "PARTYCE1.BIN" }}
			},

			{
				Name="Matilda Gate Fix",
				Description="Makes the Matilda Gate immovable.",
				Active=1,
				Files = {{ Type = "PPF", PatchFileName = "matilda_gate.ppf", GameFileName = "VI11.BIN" }}
			},

			{
				Name="Rune Unite Fix",
				Description="Repairs a bug that could allow a single character to participate in multiple Rune Unite spells.",
				Active=1,
				Files = {{ Type= "PPF", PatchFileName = "rune_unite_1.ppf", GameFileName = "BP0_FST.BIN" },
						{ Type= "PPF", PatchFileName = "rune_unite_2.ppf", GameFileName = "BP0_SEC.BIN" }}
			},

			{
				Name="Scroll Shop Display Fix",
				Description="Stops the Scroll Shop from displaying the actual Names of unidentified items.",
				Active=1,
				Files = {{ Type="PPF", PatchFileName = "scroll_shop.ppf", GameFileName = "FUDAZUK.BIN" }}
			},

			{
				Name="Tenzan Pass Enemies Fix",
				Description="Translates the Names of the enemies in Tenzan pass, and adds the formations that contain the Chimera-type monster to the encounter table.",
				Active=1,
				Files = {{ Type="PPF", PatchFileName = "tenzan.ppf", GameFileName = "VA11.BIN" }}
			},
			
			{
				Name="Two River Crash Fix",
				Description="Fixes a translation issue that can cause the game to crash during the skirmish with Kiba's forces outside Two River.",
				Active=1,
				Files = {{ Type= "PPF", PatchFileName = "tworivercrash_VH01.ppf", GameFileName = "VH01.BIN" },
						 { Type= "PPF", PatchFileName = "tworivercrash_VH06.ppf", GameFileName = "VH06.BIN" },
						 { Type= "PPF", PatchFileName = "tworivercrash_VH07.ppf", GameFileName = "VH07.BIN" },
						 { Type= "PPF", PatchFileName = "tworivercrash_VH08.ppf", GameFileName = "VH08.BIN" },
						 { Type= "PPF", PatchFileName = "tworivercrash_VH11.ppf", GameFileName = "VH11.BIN" },
						 { Type= "PPF", PatchFileName = "tworivercrash_VH12.ppf", GameFileName = "VH12.BIN" }
						}
			}
		  }, -- end patches
		  LBAPatchList = {
			{ name="SLES_024.44",     path="/",     startsector=0,   seekpos=0x7dfc8 },
			{ name="VJ24.BIN",     path="/CDROM/100_ARJ/",     startsector=0,   seekpos=0x23f24 },
			{ name="FURO.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x16e1c },
			{ name="HSOUND.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x3cd4 },
			{ name="RBATTLE.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x4365c },
			{ name="SNDTEST.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x28118 },
			{ name="IPRG_A3.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11508 },
			{ name="IPRG_A4.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11f58 },
			{ name="IPRG_B2.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10890 },
			{ name="IPRG_D.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10e24 },
			{ name="IPRG_D2.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10e48 },
			{ name="IPRG_D3.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11368 },
			{ name="IPRG_J.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10ce8 },
			{ name="BOOT.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x252a0 },
			{ name="ENDING.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x3c54 },
			{ name="OVER.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0xbb70 },
			{ name="STAFF.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x3060 }
		  }, --end LBAPatchList
		  FileList = {
			{ name="SYSTEM.CNF",     path="/",     lba=0,   size=0 },
			{ name="SLES_024.44",     path="/",     lba=0,   size=0 },
			{ name="BGM.XA",     path="/CDROM/",     lba=0,   size=0 },
			{ name="A_1.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="A_2.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="BGM.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="BGM2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="CYUTON.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="CYUUTON.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="KYARO.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="KYARO1.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="ROUYA.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="ROUYA.TKD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="SEQ.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="SEQ2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_B.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_C.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_D.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_E.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_F.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_G.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_H.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA01.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA02.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA03.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA04.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA05.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA06.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA07.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA08.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA09.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA10.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA11.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA12.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA13.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA14.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA15.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA16.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA17.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA18.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA19.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA20.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA21.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA22.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA23.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA24.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA25.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA26.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="WA.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO1.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO21.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO3.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="B_1.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="B_3.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="DANGION.TKD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="ENHOKU1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="ENHOK_CB.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="IKKI.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="KAISOU4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="SD_B01.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO5.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB01.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB02.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB03.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB04.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB05.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB06.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB07.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB08.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB09.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB10.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB11.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB12.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB13.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB14.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB15.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB16.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB17.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB18.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB19.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB20.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB21.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK5.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK6.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK7.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="WB.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="YOUHEI.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="YUUWAKU.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="C_1.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="C_4.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="C_5.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="FCEV_C.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="KOKKYO.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="KORONE.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_10.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_11.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_5.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_6.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_7.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_8.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_81.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_9.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_5.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE3_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE3_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="SD_C05.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC01.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC02.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC03.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC04.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC05.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC06.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC07.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC08.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC09.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC10.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC11.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC12.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC13.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC14.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC15.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC16.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC17.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC18.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC19.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC20.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC21.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC22.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC23.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC24.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC25.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC26.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC27.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC28.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC29.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC30.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC31.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="WC.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="D_1.VAB",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KAZE1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KAZE2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KUSU1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KUSUKUSU.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH4.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH5.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH6.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RADATO1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RADATO2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RADATO3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RUKAMOR1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RUKAMOR2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RUKAMOR3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SD_D04.VAB",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SOUTH1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SOUTH2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SOUTH3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD01.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD02.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD03.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD04.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD05.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD06.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD07.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD08.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD09.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD10.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD11.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD12.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD13.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD14.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD15.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD16.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD17.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD18.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD19.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD20.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="WD.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="E_1.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_2.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_3.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_4.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_5.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_6.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_1.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_2.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_3.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_4.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_5.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_6.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_7.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_8.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_9.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_1.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_2.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_3.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_4.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_5.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_6.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_7.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="MORIMURA.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="SEKISYO.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE01.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE02.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE03.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE04.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE05.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE06.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE07.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE08.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE09.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE10.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE11.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE12.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE13.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE14.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE15.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE16.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE17.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="WE.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="BANAR.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="BANAR2.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="ROKKAKU.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="TORAN1.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="TORAN2.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF01.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF02.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF03.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF04.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF05.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF06.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF07.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="WF.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="G_1.VAB",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KITADO.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KITADOUK.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KOKOU.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU1.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU2.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU3.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU4.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU5.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="RYUUKO.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="SD_G11.VAB",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="SOUTOU.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO1.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO2.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO21.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO3.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO4.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO5.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO6.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO7.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO8.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG01.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG02.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG03.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG04.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG05.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG06.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG07.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG08.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG09.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG10.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG11.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG12.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG13.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG14.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG15.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG16.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG17.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG18.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="WG.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="H_1.VAB",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="KOBORUT.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="LAKEWST1.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV1.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV2.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV21.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV3.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV4.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV5.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH01.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH02.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH03.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH04.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH05.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH06.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH07.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH08.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH09.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH10.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH11.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH12.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH13.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH14.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH15.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH16.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH17.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="WH.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="I_1.VAB",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="I_3.VAB",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIMURA.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU1.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU2.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU3.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU4.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU5.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU6.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU7.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU8.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU9.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="RAKUTEI.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="RAKUTEI2.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI01.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI02.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI03.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI04.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI05.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI06.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI07.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI08.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI09.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI10.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI11.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI12.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI13.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI14.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI15.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="WI.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="ANSATU1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="ANSATU2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HILAND1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HILAND2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HILAND3.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HIMURA.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="JOUI1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="J_2.VAB",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="J_3.VAB",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="LASTBOSS.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO3.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO4.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO5.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO6.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO7.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="TAIKETU1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ01.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ02.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ03.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ04.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ05.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ06.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ07.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ08.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ09.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ10.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ11.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ12.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ13.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ14.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ15.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ16.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ17.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ18.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ19.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ20.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ21.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ22.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ23.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ24.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="WAHEI1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="WJ.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HONKYO1.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO10.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO2.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO3.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO4.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO5.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO6.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO7.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO8.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO9.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI1.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI2.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI3.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI4.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI5.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI6.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="RYOURI.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_K01.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_K29.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG1.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG2.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG3.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG4.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG5.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG6.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG7.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK01.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK02.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK03.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK04.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK05.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK06.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK07.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK08.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK09.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK10.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK11.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK12.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK13.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK14.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK15.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK16.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK17.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK18.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK19.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK20.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK21.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK22.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK23.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK24.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK25.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK26.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK27.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK28.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK29.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK30.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK31.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK32.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK33.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK34.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK35.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK36.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK37.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK38.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="WK.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="ADA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AIR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AIR_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ALX_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AMD.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AMD_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ANI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="APP_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="BIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="BOR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="BOZ.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="CRY.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="CYK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="CYK_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ERI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FIC_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FUT.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GAB.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GEO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GJM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GRE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GRI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HAW.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HNF.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="JIJ.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="JO2.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="JOI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KAL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KAN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KIB_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KKL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KKS.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KLK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KMU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KNI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KRU_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KSD.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KSM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KSM_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KYU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KYU_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="LFI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="LKK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MCR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MCR_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MDO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MFI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MI2.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MIL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MRL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MSM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTF.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTS.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTT.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MUK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="NAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="NAN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="NIA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ORA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="PEC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="PR2_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="PRK_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RIK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RKL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ROL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RYE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RYN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SEE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SEE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SHE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SHU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SMB.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SNA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SRO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SSK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="STA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="STK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TAI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TAI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TEL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TOU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="UNI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VAL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VAL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VIC_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="WIG.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="WOL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZAI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZAI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZAM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZKO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZSH.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="DOUGUYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="DOUGUYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KAJIYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KAJIYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KANTEIY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KANTEIY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KOUEKI1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KOUEKI2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KPARTY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KPARTY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="MONSYOY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="MONSYOY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="PARTYIN1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="PARTYIN2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="SAVE1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="SAVE2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="UWASA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="UWASA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="YADOYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="YADOYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="BOOK.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="CHIE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="CHIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="CHIN2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="FISHING.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="FUDAZUK.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="FURO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT1.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT3.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT4.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT5.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HDOUGUYA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HKAJIYA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HMAP.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HMAP2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HMONSYOY.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HSOUND.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HWINDOW.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="KIKORI.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="MEYASU.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="MOGURA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="ODORIKO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PARTYCE1.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PARTYCH2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PARTYCHG.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PEEPING.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RBATTLE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA00.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA01.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA02.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA03.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA04.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA05.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA06.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA07.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA08.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA09.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA10.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="REST.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SHUGO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SNDTEST.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SOUKO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="STONE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SYUGOITM.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TANTEI.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TEAMCHG.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TEAMCHG2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TELEPO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="VB19PIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="VG08PIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="WAKEUP.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="WAKEUP2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="BP0_AFT.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_BEF.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_FST.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_MAK.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_SEC.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BF_DAT.BIN",     path="/CDROM/160_BFDA/",     lba=0,   size=0 },
			{ name="UD0_UN01.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN02.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN03.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN04.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN05.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN06.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN07.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN08.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN09.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN10.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN11.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN12.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN13.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN14.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN15.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN16.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN17.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN18.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN19.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN20.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN21.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN22.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN23.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN24.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN25.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN26.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN27.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN28.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN29.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN30.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN31.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN32.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN33.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN34.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN35.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN36.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN37.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN38.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN39.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN40.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN41.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN42.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN43.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN44.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN45.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN46.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN47.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN48.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN49.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN50.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN51.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="MAGIC.BAK",     path="/CDROM/180_MAGI/",     lba=0,   size=0 },
			{ name="MAGIC.VAB",     path="/CDROM/180_MAGI/",     lba=0,   size=0 },
			{ name="MD0_BLN1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BLN2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BLN3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BOR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BUR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FUR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GRI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GRI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GRI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KAE.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KAL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KKS.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KLK1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KLK2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KLK3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KSM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_LFI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MDO.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MFI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MIL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MSM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_ORA.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_RAI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SEN.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SIN.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP10.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP7.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP8.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP9.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SUI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SYO.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_UNI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_UNI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_UNI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_VAL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WINR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WOL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_ZAM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC10.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC11.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC12.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC13.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC14.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC15.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC16.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC17.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC18.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC19.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC20.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC7.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC8.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC9.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MF_BOA0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_CIM0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_CIM1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_CIM2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COC0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COC1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE6.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE7.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE8.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE9.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DREA.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DREB.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DREC.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRED.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_EAR4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GAR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GAR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GEK0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GEK1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GWO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GWO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_HAP0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_HAP1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_IMO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_IMO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_IMO2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KI20.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KI21.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KUL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KUL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS6.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS7.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LUS0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MEI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MEI1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NEC0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NEC1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NED0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NED1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_RAU0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_RAU1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SAM0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SEI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SEI1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SID0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SIE0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SIE1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SIR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_TAR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_TAR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_UNI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_WIN1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_WIN3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="METEO.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="SWORD.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="TATE.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="THUNDER.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="DOR.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="DOR_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GOL.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GOL_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GOR.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GWO1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GWO2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="HAP1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="HAP2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="HAP_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="IMO1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="IMO2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="IMO3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="KI1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="KUL_SID.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LAS.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUC1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUC2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUC3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS4.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS5.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NEC1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NEC2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NEC3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NIK.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NIK_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAT.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAT_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAU1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAU2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RYO.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RYO_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="SEI.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="SIE1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="EFFECT.BIN",     path="/CDROM/220_EFFE/",     lba=0,   size=0 },
			{ name="EB_VA05.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB02.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB02B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB02C.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB19.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC07B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC11.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC16.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC16B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC16C.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VD07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VD07B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE02.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE03.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE06.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE08.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE15.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG03.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG04.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VH01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VH08.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VJ04.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EVEFFECT.BIN",     path="/CDROM/240_EVEN/",     lba=0,   size=0 },
			{ name="FE_DAT00.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT01.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT02.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT03.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT04.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT05.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT06.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT07.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT08.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT09.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT10.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT11.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT12.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT13.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT14.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT15.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT16.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT17.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT18.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT19.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT20.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="IDAT_A3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_A4.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_B2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_D.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_D2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_D3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_J.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_A3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_A4.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_B2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_D.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_D2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_D3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_J.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="BLOAD.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="BOOT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="BOOTDAT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ENDING.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ENDING1.SDD",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ENDING2.SDD",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT4.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT5.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT6.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA4.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA5.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDAT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="G1LOAD.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="KNMLOGO.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="OVER.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="STAFF.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ST_END1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ST_END2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ST_END3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="SVINI.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ALL.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="ARMS.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="ARMSD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="ARMST.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="BOGU.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="CKD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="DISH.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="DOGU.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="EMBL.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="EX_ITEM.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="FNAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="FNO.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="FOOD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="GAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="KEVENT.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="KOTEI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="MAGI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="MS.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="NAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="PARTY.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="SUKI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="TABLE.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="TOWN.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="TRADE.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="VARIA.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="WAR_DAT.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="SD_WAR.BAK",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="SD_WAR.VAB",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_01.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_02.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_03.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_04.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_05.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_06.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_07.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_08.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_09.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_10.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_11.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_12.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_13.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_14.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_15.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_16.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="BOSS0.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="BOSS1.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="BOSS2.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO1.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO2.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO3.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO5.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO6.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="KONA5OLD.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="KONAMIC5.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="LOOP.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="OP_US.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="ROCK.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="_KONAMIC.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="ZZZ.STR",     path="/",     lba=0,   size=0 }
		  } -- end filelist
		}, --end game DE
		{ Name = "Suikoden II", Region = "Europe", Language = "Spanish", Serial = "SLES_024.45", PatchDir="patch\\European-S",
			Patches = {
			{
				Name = "Badeaux Appears",
				Description = "Badeaux will appear in the castle grounds near the pond behind the library.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "badeaux.ppf", GameFileName = "VK11.BIN" }}
			},

			{
				Name = "Castle Armory Fix",
				Description = "Prevents the Castle Armory from selling items when the player has insufficient potch.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "castle_armory.ppf", GameFileName = "HDOUGUYA.BIN" }}
			},

			{
				Name = "Castle Farm Fix",
				Description = "Fixes the Castle Farm module so the player can hand in multiple seeds or animals at one time.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "castle_farm.ppf", GameFileName = "VK19.BIN" }}
			},

			{
				Name="Chaco Bug Fix",
				Description="Adds a missing Set Level command to Chaco's recruit script.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "chaco.ppf", GameFileName = "VH01.BIN" }}
			},

			{
				Name="Circlet Name Correction",
				Description="Changes \"circuret\" to \"circlet\".",
				Active = 1, Toggle = 0,
				Search = {{ Value="CIRCURET", Replace="CIRCLET", Case="false", VarQual="circlet" }}
			},

			{
				Name="Collectibles Fix",
				Description="Repairs errors in the after-battle module that can prevent Recipes, Plans, Window Sets and Sound Sets from dropping.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "collectibles.ppf", GameFileName = "BP0_AFT.BIN" }}
			},

			{
				Name="Forgiver Sign Fix",
				Description="Fix display.  Forgiver Sign does zero damage, but a huge value is displayed.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "forgiver_sign_1.ppf", GameFileName = "BP0_FST.BIN" },
						{ Type = "PPF", PatchFileName = "forgiver_sign_2.ppf", GameFileName = "BP0_SEC.BIN" }}
			},

			{
				Name="Godspeed Rune Name Correction",
				Description="Changes \"True Holy\" to \"Godspeed\".",
				Active = 1, Toggle = 0,
				Search = {{ Value="True Holy", Replace="GODSPEED", Case="false", VarQual="godspeed" }}
			},

			{
				Name="Gozz Rune Renamed Minotaur Rune",
				Description="Changes \"Gozz\" to \"Minotaur\".",
				Active = 1, Toggle = 0,
				Search = {{ Value="GOZZ", Replace="MINOTAUR", Case="false", VarQual="gozz" }},
				Excludes = { "Gozz Rune Renamed Gozu Rune" }
			},

			{
				Name="Gozz Rune Renamed Gozu Rune",
				Description="Changes \"Gozz\" to \"Gozu\".",
				Active = 0,
				Search = {{ Value="GOZZ", Replace="GOZU", Case="false", VarQual="gozz" }},
				Excludes = { "Gozz Rune Renamed Minotaur Rune" }
			},

			{
				Name="Inns Fix",
				Description="Prevent the Inn from healing the party for free.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "inn_1.ppf", GameFileName = "YADOYA1.BIN" },
						{ Type = "PPF", PatchFileName = "inn_2.ppf", GameFileName = "YADOYA2.BIN" }}
			},

			{
				Name="Join At 99 Fix",
				Description="Prevents recruits from joining at level 99 in error.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "join_at_99.ppf", GameFileName = "SLES_024.45" }}
			},

			{
				Name="Kindness Rune Fix (No Penalty)",
				Description="When a character's invisible Kindness Rating goes below zero the Rune gives no bonus to attack instead of maxing it out.",
				Active = 0,
				Files = {{ Type="PPF", PatchFileName = "kindness_rune_np.ppf", GameFileName = "SLES_024.45" }},
				Excludes = { "Kindness Rune Fix (Penalty)" }
			},

			{
				Name="Kindness Rune Fix (Penalty)",
				Description="When a character's invisible Kindness Rating goes below zero the Rune gives a penalty to attack instead of maxing it out.",
				Active=1,
				Files = {{ Type="PPF", PatchFileName = "kindness_rune.ppf", GameFileName = "SLES_024.45" }},
				Excludes = { "Kindness Rune Fix (No Penalty)" }
			},

			{
				Name="Knife-thrower Fix",
				Description="Adds a custom routine for the Knife-throwing event in Ryube that keeps HP from being zeroed.",
				Active=1,
				Files = {{ Type = "PPF", PatchFileName = "knife_thrower.ppf", GameFileName = "VB18.BIN" }}
			},

			{
				Name="Lamb Fix (Unicorn Woods)",
				Description="Separates the acquired flag for the lamb from the flag for the first chest (can now get both).",
				Active=1,
				Files = {{ Type= "PPF", PatchFileName = "lamb.ppf", GameFileName = "VH10.BIN" }}
			},

			{
				Name="Luca Battle Party-change Fix",
				Description="The party is emptied but not reformed prior to fighting Luca, allowing the hero to be alone in the back row and effectively invincible.  Fix forces the party to reform.",
				Active=1,
				Files = {{ Type = "PPF", PatchFileName = "luca.ppf", GameFileName = "PARTYCE1.BIN" }}
			},

			{
				Name="Matilda Gate Fix",
				Description="Makes the Matilda Gate immovable.",
				Active=1,
				Files = {{ Type = "PPF", PatchFileName = "matilda_gate.ppf", GameFileName = "VI11.BIN" }}
			},

			{
				Name="Rune Unite Fix",
				Description="Repairs a bug that could allow a single character to participate in multiple Rune Unite spells.",
				Active=1,
				Files = {{ Type= "PPF", PatchFileName = "rune_unite_1.ppf", GameFileName = "BP0_FST.BIN" },
						{ Type= "PPF", PatchFileName = "rune_unite_2.ppf", GameFileName = "BP0_SEC.BIN" }}
			},

			{
				Name="Scroll Shop Display Fix",
				Description="Stops the Scroll Shop from displaying the actual Names of unidentified items.",
				Active=1,
				Files = {{ Type="PPF", PatchFileName = "scroll_shop.ppf", GameFileName = "FUDAZUK.BIN" }}
			},

			{
				Name="Tenzan Pass Enemies Fix",
				Description="Translates the Names of the enemies in Tenzan pass, and adds the formations that contain the Chimera-type monster to the encounter table.",
				Active=1,
				Files = {{ Type="PPF", PatchFileName = "tenzan.ppf", GameFileName = "VA11.BIN" }}
			}
		  }, --end patches
		  LBAPatchList = {
			 { name="SLES_024.45",     path="/",     startsector=0,   seekpos=0x7dfec },
			 { name="VJ24.BIN",     path="/CDROM/100_ARJ/",     startsector=0,   seekpos=0x23f24 },
			 { name="FURO.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x16e44 },
			 { name="HSOUND.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x3ce0 },
			 { name="RBATTLE.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x43274 },
			 { name="SNDTEST.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x28104 },
			 { name="IPRG_A3.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x114f8 },
			 { name="IPRG_A4.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11f48 },
			 { name="IPRG_B2.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10880 },
			 { name="IPRG_D.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10e14 },
			 { name="IPRG_D2.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10e38 },
			 { name="IPRG_D3.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11358 },
			 { name="IPRG_J.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10cd8 },
			 { name="BOOT.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x25298 },
			 { name="ENDING.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x3c54 },
			 { name="OVER.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0xbb84 },
			 { name="STAFF.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x3060 }
		  }, --end LBAPatchList
		  FileList = {
			{ name="SYSTEM.CNF",     path="/",     lba=0,   size=0 },
			{ name="SLES_024.45",     path="/",     lba=0,   size=0 },
			{ name="BGM.XA",     path="/CDROM/",     lba=0,   size=0 },
			{ name="A_1.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="A_2.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="BGM.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="BGM2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="CYUTON.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="CYUUTON.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="KYARO.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="KYARO1.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="ROUYA.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="ROUYA.TKD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="SEQ.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="SEQ2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_B.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_C.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_D.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_E.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_F.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_G.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_H.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA01.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA02.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA03.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA04.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA05.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA06.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA07.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA08.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA09.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA10.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA11.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA12.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA13.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA14.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA15.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA16.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA17.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA18.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA19.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA20.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA21.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA22.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA23.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA24.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA25.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA26.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="WA.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO1.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO21.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO3.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="B_1.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="B_3.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="DANGION.TKD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="ENHOKU1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="ENHOK_CB.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="IKKI.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="KAISOU4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="SD_B01.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO5.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB01.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB02.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB03.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB04.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB05.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB06.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB07.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB08.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB09.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB10.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB11.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB12.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB13.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB14.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB15.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB16.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB17.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB18.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB19.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB20.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB21.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK5.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK6.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK7.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="WB.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="YOUHEI.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="YUUWAKU.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="C_1.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="C_4.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="C_5.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="FCEV_C.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="KOKKYO.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="KORONE.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_10.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_11.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_5.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_6.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_7.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_8.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_81.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_9.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_5.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE3_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE3_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="SD_C05.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC01.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC02.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC03.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC04.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC05.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC06.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC07.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC08.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC09.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC10.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC11.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC12.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC13.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC14.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC15.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC16.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC17.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC18.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC19.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC20.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC21.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC22.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC23.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC24.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC25.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC26.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC27.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC28.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC29.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC30.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC31.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="WC.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="D_1.VAB",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KAZE1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KAZE2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KUSU1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KUSUKUSU.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH4.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH5.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH6.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RADATO1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RADATO2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RADATO3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RUKAMOR1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RUKAMOR2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RUKAMOR3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SD_D04.VAB",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SOUTH1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SOUTH2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SOUTH3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD01.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD02.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD03.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD04.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD05.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD06.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD07.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD08.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD09.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD10.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD11.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD12.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD13.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD14.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD15.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD16.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD17.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD18.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD19.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD20.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="WD.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="E_1.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_2.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_3.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_4.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_5.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_6.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_1.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_2.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_3.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_4.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_5.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_6.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_7.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_8.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_9.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_1.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_2.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_3.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_4.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_5.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_6.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_7.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="MORIMURA.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="SEKISYO.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE01.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE02.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE03.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE04.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE05.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE06.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE07.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE08.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE09.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE10.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE11.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE12.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE13.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE14.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE15.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE16.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE17.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="WE.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="BANAR.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="BANAR2.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="ROKKAKU.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="TORAN1.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="TORAN2.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF01.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF02.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF03.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF04.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF05.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF06.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF07.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="WF.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="G_1.VAB",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KITADO.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KITADOUK.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KOKOU.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU1.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU2.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU3.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU4.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU5.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="RYUUKO.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="SD_G11.VAB",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="SOUTOU.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO1.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO2.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO21.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO3.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO4.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO5.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO6.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO7.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO8.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG01.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG02.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG03.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG04.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG05.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG06.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG07.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG08.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG09.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG10.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG11.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG12.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG13.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG14.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG15.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG16.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG17.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG18.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="WG.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="H_1.VAB",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="KOBORUT.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="LAKEWST1.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV1.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV2.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV21.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV3.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV4.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV5.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH01.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH02.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH03.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH04.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH05.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH06.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH07.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH08.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH09.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH10.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH11.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH12.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH13.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH14.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH15.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH16.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH17.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="WH.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="I_1.VAB",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="I_3.VAB",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIMURA.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU1.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU2.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU3.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU4.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU5.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU6.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU7.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU8.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU9.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="RAKUTEI.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="RAKUTEI2.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI01.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI02.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI03.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI04.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI05.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI06.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI07.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI08.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI09.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI10.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI11.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI12.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI13.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI14.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI15.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="WI.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="ANSATU1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="ANSATU2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HILAND1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HILAND2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HILAND3.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HIMURA.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="JOUI1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="J_2.VAB",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="J_3.VAB",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="LASTBOSS.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO3.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO4.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO5.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO6.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO7.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="TAIKETU1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ01.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ02.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ03.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ04.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ05.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ06.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ07.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ08.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ09.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ10.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ11.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ12.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ13.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ14.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ15.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ16.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ17.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ18.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ19.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ20.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ21.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ22.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ23.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ24.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="WAHEI1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="WJ.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HONKYO1.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO10.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO2.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO3.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO4.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO5.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO6.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO7.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO8.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO9.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI1.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI2.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI3.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI4.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI5.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI6.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="RYOURI.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_K01.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_K29.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG1.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG2.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG3.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG4.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG5.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG6.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG7.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK01.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK02.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK03.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK04.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK05.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK06.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK07.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK08.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK09.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK10.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK11.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK12.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK13.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK14.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK15.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK16.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK17.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK18.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK19.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK20.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK21.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK22.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK23.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK24.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK25.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK26.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK27.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK28.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK29.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK30.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK31.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK32.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK33.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK34.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK35.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK36.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK37.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK38.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="WK.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="ADA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AIR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AIR_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ALX_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AMD.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AMD_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ANI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="APP_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="BIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="BOR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="BOZ.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="CRY.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="CYK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="CYK_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ERI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FIC_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FUT.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GAB.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GEO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GJM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GRE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GRI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HAW.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HNF.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="JIJ.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="JO2.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="JOI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KAL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KAN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KIB_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KKL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KKS.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KLK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KMU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KNI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KRU_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KSD.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KSM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KSM_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KYU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KYU_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="LFI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="LKK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MCR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MCR_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MDO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MFI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MI2.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MIL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MRL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MSM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTF.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTS.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTT.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MUK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="NAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="NAN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="NIA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ORA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="PEC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="PR2_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="PRK_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RIK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RKL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ROL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RYE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RYN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SEE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SEE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SHE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SHU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SMB.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SNA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SRO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SSK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="STA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="STK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TAI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TAI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TEL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TOU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="UNI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VAL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VAL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VIC_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="WIG.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="WOL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZAI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZAI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZAM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZKO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZSH.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="DOUGUYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="DOUGUYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KAJIYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KAJIYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KANTEIY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KANTEIY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KOUEKI1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KOUEKI2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KPARTY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KPARTY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="MONSYOY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="MONSYOY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="PARTYIN1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="PARTYIN2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="SAVE1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="SAVE2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="UWASA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="UWASA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="YADOYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="YADOYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="BOOK.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="CHIE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="CHIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="CHIN2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="FISHING.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="FUDAZUK.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="FURO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT1.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT3.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT4.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT5.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HDOUGUYA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HKAJIYA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HMAP.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HMAP2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HMONSYOY.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HSOUND.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HWINDOW.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="KIKORI.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="MEYASU.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="MOGURA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="ODORIKO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PARTYCE1.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PARTYCH2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PARTYCHG.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PEEPING.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RBATTLE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA00.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA01.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA02.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA03.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA04.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA05.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA06.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA07.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA08.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA09.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA10.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="REST.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SHUGO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SNDTEST.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SOUKO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="STONE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SYUGOITM.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TANTEI.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TEAMCHG.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TEAMCHG2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TELEPO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="VB19PIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="VG08PIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="WAKEUP.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="WAKEUP2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="BP0_AFT.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_BEF.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_FST.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_MAK.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_SEC.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BF_DAT.BIN",     path="/CDROM/160_BFDA/",     lba=0,   size=0 },
			{ name="UD0_UN01.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN02.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN03.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN04.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN05.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN06.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN07.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN08.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN09.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN10.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN11.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN12.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN13.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN14.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN15.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN16.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN17.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN18.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN19.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN20.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN21.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN22.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN23.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN24.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN25.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN26.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN27.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN28.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN29.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN30.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN31.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN32.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN33.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN34.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN35.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN36.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN37.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN38.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN39.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN40.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN41.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN42.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN43.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN44.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN45.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN46.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN47.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN48.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN49.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN50.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN51.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="MAGIC.BAK",     path="/CDROM/180_MAGI/",     lba=0,   size=0 },
			{ name="MAGIC.VAB",     path="/CDROM/180_MAGI/",     lba=0,   size=0 },
			{ name="MD0_BLN1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BLN2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BLN3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BOR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BUR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FUR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GRI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GRI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GRI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KAE.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KAL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KKS.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KLK1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KLK2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KLK3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KSM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_LFI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MDO.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MFI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MIL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MSM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_ORA.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_RAI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SEN.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SIN.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP10.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP7.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP8.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP9.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SUI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SYO.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_UNI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_UNI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_UNI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_VAL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WINR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WOL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_ZAM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC10.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC11.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC12.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC13.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC14.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC15.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC16.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC17.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC18.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC19.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC20.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC7.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC8.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC9.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MF_BOA0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_CIM0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_CIM1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_CIM2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COC0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COC1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE6.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE7.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE8.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE9.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DREA.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DREB.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DREC.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRED.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_EAR4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GAR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GAR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GEK0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GEK1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GWO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GWO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_HAP0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_HAP1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_IMO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_IMO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_IMO2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KI20.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KI21.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KUL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KUL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS6.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS7.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LUS0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MEI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MEI1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NEC0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NEC1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NED0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NED1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_RAU0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_RAU1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SAM0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SEI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SEI1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SID0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SIE0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SIE1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SIR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_TAR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_TAR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_UNI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_WIN1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_WIN3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="METEO.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="SWORD.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="TATE.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="THUNDER.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="DOR.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="DOR_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GOL.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GOL_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GOR.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GWO1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GWO2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="HAP1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="HAP2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="HAP_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="IMO1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="IMO2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="IMO3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="KI1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="KUL_SID.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LAS.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUC1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUC2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUC3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS4.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS5.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NEC1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NEC2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NEC3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NIK.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NIK_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAT.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAT_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAU1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAU2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RYO.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RYO_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="SEI.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="SIE1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="EFFECT.BIN",     path="/CDROM/220_EFFE/",     lba=0,   size=0 },
			{ name="EB_VA05.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB02.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB02B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB02C.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB19.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC07B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC11.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC16.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC16B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC16C.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VD07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VD07B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE02.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE03.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE06.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE08.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE15.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG03.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG04.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VH01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VH08.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VJ04.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EVEFFECT.BIN",     path="/CDROM/240_EVEN/",     lba=0,   size=0 },
			{ name="FE_DAT00.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT01.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT02.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT03.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT04.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT05.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT06.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT07.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT08.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT09.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT10.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT11.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT12.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT13.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT14.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT15.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT16.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT17.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT18.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT19.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT20.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="IDAT_A3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_A4.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_B2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_D.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_D2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_D3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_J.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_A3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_A4.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_B2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_D.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_D2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_D3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_J.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="BLOAD.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="BOOT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="BOOTDAT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ENDING.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ENDING1.SDD",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ENDING2.SDD",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT4.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT5.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT6.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA4.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA5.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDAT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="G1LOAD.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="KNMLOGO.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="OVER.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="STAFF.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ST_END1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ST_END2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ST_END3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="SVINI.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ALL.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="ARMS.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="ARMSD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="ARMST.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="BOGU.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="CKD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="DISH.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="DOGU.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="EMBL.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="EX_ITEM.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="FNAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="FNO.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="FOOD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="GAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="KEVENT.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="KOTEI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="MAGI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="MS.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="NAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="PARTY.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="SUKI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="TABLE.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="TOWN.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="TRADE.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="VARIA.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="WAR_DAT.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="SD_WAR.BAK",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="SD_WAR.VAB",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_01.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_02.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_03.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_04.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_05.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_06.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_07.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_08.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_09.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_10.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_11.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_12.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_13.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_14.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_15.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_16.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="BOSS0.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="BOSS1.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="BOSS2.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO1.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO2.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO3.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO5.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO6.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="KONA5OLD.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="KONAMIC5.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="LOOP.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="OP_US.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="ROCK.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="_KONAMIC.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="ZZZ.STR",     path="/",     lba=0,   size=0 }
		  } --end filelist
		},
		{ Name = "Gensou Suikoden II", Region = "Japan", Language = "Japanese", Serial = "SLPM_861.68", PatchDir="patch\\Japan",
			Patches = {
			{
				Name = "Badeaux Appears",
				Description = "Badeaux will appear in the castle grounds near the pond behind the library.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "badeaux.ppf", GameFileName = "VK11.BIN" }}
			},

			{
				Name = "Castle Armory Fix",
				Description = "Prevents the Castle Armory from selling items when the player has insufficient potch.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "castle_armory.ppf", GameFileName = "HDOUGUYA.BIN" }}
			},

			{
				Name = "Castle Farm Fix",
				Description = "Fixes the Castle Farm module so the player can hand in multiple seeds or animals at one time.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "castle_farm.ppf", GameFileName = "VK19.BIN" }}
			},

			{
				Name="Chaco Bug Fix",
				Description="Adds a missing Set Level command to Chaco's recruit script.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "chaco.ppf", GameFileName = "VH01.BIN" }}
			},

			{
				Name="Collectibles Fix",
				Description="Repairs errors in the after-battle module that can prevent Recipes, Plans, Window Sets and Sound Sets from dropping.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "collectibles.ppf", GameFileName = "BP0_AFT.BIN" }}
			},

			{
				Name="Forgiver Sign Fix",
				Description="Fix display.  Forgiver Sign does zero damage, but a huge value is displayed.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "forgiver_sign_1.ppf", GameFileName = "BP0_FST.BIN" },
						{ Type = "PPF", PatchFileName = "forgiver_sign_2.ppf", GameFileName = "BP0_SEC.BIN" }}
			},

			{
				Name="Inns Fix",
				Description="Prevent the Inn from healing the party for free.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "inn_1.ppf", GameFileName = "YADOYA1.BIN" },
						{ Type = "PPF", PatchFileName = "inn_2.ppf", GameFileName = "YADOYA2.BIN" }}
			},

			{
				Name="Join At 99 Fix",
				Description="Prevents recruits from joining at level 99 in error.",
				Active = 1, Toggle = 0,
				Files = {{ Type = "PPF", PatchFileName = "join_at_99.ppf", GameFileName = "SLPM_861.68" }}
			},

			{
				Name="Kindness Rune Fix (No Penalty)",
				Description="When a character's invisible Kindness Rating goes below zero the Rune gives no bonus to attack instead of maxing it out.",
				Active = 0,
				Files = {{ Type="PPF", PatchFileName = "kindness_rune_np.ppf", GameFileName = "SLPM_861.68" }},
				Excludes = { "Kindness Rune Fix (Penalty)" }
			},

			{
				Name="Kindness Rune Fix (Penalty)",
				Description="When a character's invisible Kindness Rating goes below zero the Rune gives a penalty to attack instead of maxing it out.",
				Active=1,
				Files = {{ Type="PPF", PatchFileName = "kindness_rune.ppf", GameFileName = "SLPM_861.68" }},
				Excludes = { "Kindness Rune Fix (No Penalty)" }
			},

			{
				Name="Knife-thrower Fix",
				Description="Adds a custom routine for the Knife-throwing event in Ryube that keeps HP from being zeroed.",
				Active=1,
				Files = {{ Type = "PPF", PatchFileName = "knife_thrower.ppf", GameFileName = "VB18.BIN" }}
			},

			{
				Name="Lamb Fix (Unicorn Woods)",
				Description="Separates the acquired flag for the lamb from the flag for the first chest (can now get both).",
				Active=1,
				Files = {{ Type= "PPF", PatchFileName = "lamb.ppf", GameFileName = "VH10.BIN" }}
			},

			{
				Name="Luca Battle Party-change Fix",
				Description="The party is emptied but not reformed prior to fighting Luca, allowing the hero to be alone in the back row and effectively invincible.  Fix forces the party to reform.",
				Active=1,
				Files = {{ Type = "PPF", PatchFileName = "luca.ppf", GameFileName = "PARTYCE1.BIN" }}
			},

			{
				Name="Matilda Gate Fix",
				Description="Makes the Matilda Gate immovable.",
				Active=1,
				Files = {{ Type = "PPF", PatchFileName = "matilda_gate.ppf", GameFileName = "VI11.BIN" }}
			},

			{
				Name="Rune Unite Fix",
				Description="Repairs a bug that could allow a single character to participate in multiple Rune Unite spells.",
				Active=1,
				Files = {{ Type= "PPF", PatchFileName = "rune_unite_1.ppf", GameFileName = "BP0_FST.BIN" },
						{ Type= "PPF", PatchFileName = "rune_unite_2.ppf", GameFileName = "BP0_SEC.BIN" }}
			},

			{
				Name="Scroll Shop Display Fix",
				Description="Stops the Scroll Shop from displaying the actual Names of unidentified items.",
				Active=1,
				Files = {{ Type="PPF", PatchFileName = "scroll_shop.ppf", GameFileName = "FUDAZUK.BIN" }}
			},

			{
				Name="Tenzan Pass Enemies Fix",
				Description="Translates the Names of the enemies in Tenzan pass, and adds the formations that contain the Chimera-type monster to the encounter table.",
				Active=1,
				Files = {{ Type="PPF", PatchFileName = "tenzan.ppf", GameFileName = "VA11.BIN" }}
			}
		  }, --end patches
		  LBAPatchList = {
			{ name="SLPM_861.68",     path="/",     startsector=0,   seekpos=0x7e10c },
			{ name="VJ24.BIN",     path="/CDROM/100_ARJ/",     startsector=0,   seekpos=0x23f84 },
			{ name="FURO.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x16b9c },
			{ name="HSOUND.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x3c20 },
			{ name="RBATTLE.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x41dac },
			{ name="SNDTEST.BIN",     path="/CDROM/140_HONP/",     startsector=0,   seekpos=0x27e7c },
			{ name="IPRG_A3.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x114c8 },
			{ name="IPRG_A4.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11f18 },
			{ name="IPRG_B2.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10850 },
			{ name="IPRG_D.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10de4 },
			{ name="IPRG_D2.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10e08 },
			{ name="IPRG_D3.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x11328 },
			{ name="IPRG_J.BIN",     path="/CDROM/260_IKKI/",     startsector=0,   seekpos=0x10ca8 },
			{ name="BOOT.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x236a4 },
			{ name="ENDING.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x3d0c },
			{ name="OVER.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0xb9c4 },
			{ name="STAFF.BIN",     path="/CDROM/270_BOOT/",     startsector=0,   seekpos=0x2fc0 }
		  }, --end LBAPatchList
		  FileList = {
			{ name="SLPM_861.68",     path="/",     lba=0,   size=0 },
			{ name="SYSTEM.CNF",     path="/",     lba=0,   size=0 },
			{ name="BGM.XA",     path="/CDROM/",     lba=0,   size=0 },
			{ name="A_1.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="A_2.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="BGM.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="BGM2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="CYUTON.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="CYUUTON.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="KYARO.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="KYARO1.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="ROUYA.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="ROUYA.TKD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="SEQ.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="SEQ2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_B.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_C.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_D.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_E.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_F.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_G.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="STAY_H.VAB",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA01.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA02.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA03.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA04.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA05.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA06.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA07.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA08.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA09.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA10.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA11.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA12.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA13.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA14.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA15.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA16.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA17.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA18.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA19.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA20.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA21.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA22.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA23.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA24.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA25.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="VA26.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="WA.BIN",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO1.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO2.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO21.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="YAKUSO3.SDD",     path="/CDROM/010_ARA/",     lba=0,   size=0 },
			{ name="B_1.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="B_3.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="DANGION.TKD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="ENHOKU1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="ENHOK_CB.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="IKKI.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="KAISOU4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="RYUBE4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="SD_B01.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="TOTO5.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB01.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB02.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB03.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB04.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB05.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB06.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB07.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB08.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB09.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB10.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB11.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB12.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB13.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB14.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB15.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB16.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB17.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB18.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB19.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB20.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VB21.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK1.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK2.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK3.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK4.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK5.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK6.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="VIK7.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="WB.BIN",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="YOUHEI.VAB",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="YUUWAKU.SDD",     path="/CDROM/020_ARB/",     lba=0,   size=0 },
			{ name="C_1.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="C_4.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="C_5.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="FCEV_C.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="HAKUSHI4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="KOKKYO.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="KORONE.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_10.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_11.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_5.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_6.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_7.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_8.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_81.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE1_9.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_4.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE2_5.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE3.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE3_1.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="MUSE3_2.SDD",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="SD_C05.VAB",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC01.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC02.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC03.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC04.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC05.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC06.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC07.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC08.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC09.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC10.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC11.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC12.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC13.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC14.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC15.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC16.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC17.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC18.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC19.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC20.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC21.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC22.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC23.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC24.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC25.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC26.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC27.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC28.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC29.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC30.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="VC31.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="WC.BIN",     path="/CDROM/030_ARC/",     lba=0,   size=0 },
			{ name="D_1.VAB",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KAZE1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KAZE2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KUSU1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="KUSUKUSU.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH4.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH5.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="NORTH6.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RADATO1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RADATO2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RADATO3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RUKAMOR1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RUKAMOR2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="RUKAMOR3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SD_D04.VAB",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SOUTH1.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SOUTH2.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="SOUTH3.SDD",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD01.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD02.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD03.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD04.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD05.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD06.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD07.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD08.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD09.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD10.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD11.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD12.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD13.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD14.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD15.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD16.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD17.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD18.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD19.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="VD20.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="WD.BIN",     path="/CDROM/040_ARD/",     lba=0,   size=0 },
			{ name="E_1.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_2.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_3.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_4.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_5.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="E_6.VAB",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_1.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_2.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_3.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_4.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_5.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_6.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_7.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_8.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN1_9.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_1.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_2.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_3.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_4.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_5.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_6.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="GREEN2_7.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="MORIMURA.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="SEKISYO.SDD",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE01.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE02.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE03.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE04.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE05.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE06.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE07.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE08.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE09.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE10.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE11.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE12.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE13.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE14.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE15.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE16.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="VE17.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="WE.BIN",     path="/CDROM/050_ARE/",     lba=0,   size=0 },
			{ name="BANAR.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="BANAR2.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="ROKKAKU.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="TORAN1.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="TORAN2.SDD",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF01.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF02.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF03.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF04.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF05.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF06.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="VF07.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="WF.BIN",     path="/CDROM/060_ARF/",     lba=0,   size=0 },
			{ name="G_1.VAB",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KITADO.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KITADOUK.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KOKOU.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU1.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU2.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU3.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU4.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="KUROMU5.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="RYUUKO.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="SD_G11.VAB",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="SOUTOU.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO1.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO2.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO21.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO3.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO4.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO5.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO6.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO7.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="TINTO8.SDD",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG01.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG02.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG03.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG04.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG05.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG06.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG07.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG08.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG09.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG10.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG11.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG12.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG13.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG14.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG15.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG16.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="VG17.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			--{ name="VG18.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="WG.BIN",     path="/CDROM/070_ARG/",     lba=0,   size=0 },
			{ name="H_1.VAB",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="KOBORUT.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="LAKEWST1.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV1.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV2.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV21.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV3.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV4.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="TWORIV5.SDD",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH01.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH02.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH03.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH04.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH05.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH06.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH07.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH08.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH09.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH10.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH11.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="VH12.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			--{ name="VH13.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			--{ name="VH14.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			--{ name="VH15.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			--{ name="VH16.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			--{ name="VH17.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="WH.BIN",     path="/CDROM/080_ARH/",     lba=0,   size=0 },
			{ name="I_1.VAB",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="I_3.VAB",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIMURA.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU1.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU2.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU3.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU4.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU5.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU6.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU7.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU8.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="MATIRU9.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="RAKUTEI.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="RAKUTEI2.SDD",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI01.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI02.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI03.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI04.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI05.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI06.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI07.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI08.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI09.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI10.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI11.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI12.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI13.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI14.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="VI15.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="WI.BIN",     path="/CDROM/090_ARI/",     lba=0,   size=0 },
			{ name="ANSATU1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="ANSATU2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HILAND1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HILAND2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HILAND3.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HIMURA.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="JOUI1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="J_2.VAB",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="J_3.VAB",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="LASTBOSS.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO2.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO3.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO4.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO5.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO6.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="RURUNO7.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="TAIKETU1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ01.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ02.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ03.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ04.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ05.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ06.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ07.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ08.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ09.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ10.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ11.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ12.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ13.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ14.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ15.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ16.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ17.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ18.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ19.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ20.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ21.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ22.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ23.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="VJ24.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="WAHEI1.SDD",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="WJ.BIN",     path="/CDROM/100_ARJ/",     lba=0,   size=0 },
			{ name="HONKYO1.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO10.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO2.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO3.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO4.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO5.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO6.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO7.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO8.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="HONKYO9.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI1.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI2.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI3.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI4.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI5.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="MINI6.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="RYOURI.SDD",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_K01.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_K29.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG1.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG2.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG3.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG4.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG5.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG6.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="SD_MG7.VAB",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK01.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK02.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK03.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK04.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK05.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK06.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK07.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK08.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK09.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK10.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK11.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK12.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK13.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK14.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK15.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK16.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK17.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK18.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK19.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK20.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK21.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK22.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK23.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK24.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK25.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK26.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK27.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK28.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK29.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK30.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK31.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK32.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK33.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK34.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK35.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK36.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="VK37.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="WK.BIN",     path="/CDROM/110_ARK/",     lba=0,   size=0 },
			{ name="ADA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AIR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AIR_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ALX_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AMD.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="AMD_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ANI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="APP_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="BIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="BOR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="BOZ.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="CRY.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="CYK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="CYK_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ERI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FIC_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FRK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="FUT.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GAB.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GEO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GJM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GRE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="GRI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HAW.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="HNF.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="JIJ.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="JO2.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="JOI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KAL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KAN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KIB_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KKL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KKS.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KLK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KMU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KNI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KRU_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KSD.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KSM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KSM_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KYU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="KYU_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="LFI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="LKK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MCR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MCR_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MDO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MFI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MI2.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MIL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MRL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MSM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTF.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTS.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MTT.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="MUK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="NAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="NAN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="NIA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ORA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="PEC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="PR2_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="PRK_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RIK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RKL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ROL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RYE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="RYN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SEE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SEE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SHE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SHU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIE.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIE_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIN_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SIR.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SMB.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SNA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SRO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="SSK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="STA.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="STK.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TAI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TAI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TEL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TEN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="TOU.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="UNI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VAL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VAL_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VAN.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VIC.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="VIC_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="WIG.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="WOL.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZAI.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZAI_D.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZAM.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZKO.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="ZSH.BIN",     path="/CDROM/120_PLAY/",     lba=0,   size=0 },
			{ name="DOUGUYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="DOUGUYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KAJIYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KAJIYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KANTEIY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KANTEIY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KOUEKI1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KOUEKI2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KPARTY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="KPARTY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="MONSYOY1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="MONSYOY2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="PARTYIN1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="PARTYIN2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="SAVE1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="SAVE2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="UWASA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="UWASA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="YADOYA1.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="YADOYA2.BIN",     path="/CDROM/130_SHOP/",     lba=0,   size=0 },
			{ name="BOOK.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="CHIE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="CHIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="CHIN2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="FISHING.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="FUDAZUK.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="FURO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT1.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT3.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT4.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="F_DAT5.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HDOUGUYA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HKAJIYA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HMAP.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HMAP2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HMONSYOY.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HSOUND.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="HWINDOW.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="KIKORI.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="MEYASU.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="MOGURA.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="ODORIKO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PARTYCE1.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PARTYCH2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PARTYCHG.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="PEEPING.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RBATTLE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA00.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA01.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA02.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA03.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA04.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA05.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA06.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA07.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA08.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA09.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="RB_CHA10.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="REST.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SHUGO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SNDTEST.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SOUKO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="STONE.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="SYUGOITM.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TANTEI.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TEAMCHG.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TEAMCHG2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="TELEPO.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="VB19PIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="VG08PIN.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="WAKEUP.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="WAKEUP2.BIN",     path="/CDROM/140_HONP/",     lba=0,   size=0 },
			{ name="BP0_AFT.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_BEF.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_FST.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_MAK.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BP0_SEC.BIN",     path="/CDROM/150_BPRG/BUFF0/",     lba=0,   size=0 },
			{ name="BF_DAT.BIN",     path="/CDROM/160_BFDA/",     lba=0,   size=0 },
			{ name="UD0_UN01.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN02.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN03.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN04.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN05.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN06.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN07.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN08.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN09.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN10.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN11.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN12.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN13.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN14.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN15.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN16.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN17.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN18.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN19.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN20.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN21.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN22.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN23.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN24.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN25.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN26.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN27.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN28.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN29.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN30.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN31.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN32.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN33.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN34.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN35.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN36.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN37.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN38.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN39.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN40.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN41.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN42.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN43.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN44.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN45.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN46.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN47.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN48.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN49.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN50.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="UD0_UN51.BIN",     path="/CDROM/170_UNIT/BUFF0/",     lba=0,   size=0 },
			{ name="MAGIC.BAK",     path="/CDROM/180_MAGI/",     lba=0,   size=0 },
			{ name="MAGIC.VAB",     path="/CDROM/180_MAGI/",     lba=0,   size=0 },
			{ name="MD0_BLN1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BLN2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BLN3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BOR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_BUR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_DAR6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_EAT5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FIR5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_FUR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GRI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GRI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_GRI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_HOL4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KAE.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KAL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KKS.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KLK1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KLK2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KLK3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_KSM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_LFI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MDO.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MFI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MIL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_MSM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_ORA.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_RAI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SEN.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SHI4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SIN.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP10.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP7.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP8.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SP9.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SUI.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SWO4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_SYO.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_THU5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_UNI1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_UNI2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_UNI3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_VAL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WAT5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WIN5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WINR.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_WOL.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD0_ZAM.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC1.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC10.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC11.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC12.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC13.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC14.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC15.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC16.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC17.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC18.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC19.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC2.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC20.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC3.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC4.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC5.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC6.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC7.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC8.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MD_SPC9.BIN",     path="/CDROM/180_MAGI/BUFF0/",     lba=0,   size=0 },
			{ name="MF_BOA0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_CIM0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_CIM1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_CIM2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COC0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COC1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_COL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE6.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE7.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE8.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRE9.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DREA.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DREB.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DREC.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_DRED.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_EAR4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_FIR5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GAR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GAR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GEK0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GEK1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GOR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GWO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_GWO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_HAP0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_HAP1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_IMO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_IMO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_IMO2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KI20.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KI21.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KUL0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_KUL1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS4.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS5.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS6.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LAS7.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_LUS0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG2.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAG3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAO0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MAO1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MEI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_MEI1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NEC0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NEC1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NED0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_NED1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_RAU0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_RAU1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SAM0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SEI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SEI1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SID0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SIE0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SIE1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_SIR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_TAR0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_TAR1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_UNI0.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_WIN1.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="MF_WIN3.BIN",     path="/CDROM/190_MEFE/",     lba=0,   size=0 },
			{ name="METEO.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="SWORD.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="TATE.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="THUNDER.STR",     path="/CDROM/200_MOV2/",     lba=0,   size=0 },
			{ name="DOR.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="DOR_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GOL.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GOL_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GOR.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GWO1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="GWO2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="HAP1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="HAP2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="HAP_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="IMO1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="IMO2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="IMO3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="KI1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="KUL_SID.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LAS.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUC1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUC2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUC3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS4.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="LUS5.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NEC1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NEC2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NEC3.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NIK.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="NIK_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAT.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAT_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAU1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RAU2.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RYO.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="RYO_K.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="SEI.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="SIE1.BIN",     path="/CDROM/210_BOSS/",     lba=0,   size=0 },
			{ name="EFFECT.BIN",     path="/CDROM/220_EFFE/",     lba=0,   size=0 },
			{ name="EB_VA05.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB02.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB02B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB02C.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VB19.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC07B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC11.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC16.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC16B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VC16C.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VD07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VD07B.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE02.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE03.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE06.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE08.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VE15.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG03.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG04.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VG07.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VH01.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VH08.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EB_VJ04.BIN",     path="/CDROM/230_EVBA/",     lba=0,   size=0 },
			{ name="EVEFFECT.BIN",     path="/CDROM/240_EVEN/",     lba=0,   size=0 },
			{ name="FE_DAT00.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT01.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT02.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT03.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT04.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT05.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT06.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT07.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT08.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT09.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT10.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT11.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT12.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT13.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT14.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT15.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT16.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT17.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT18.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT19.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="FE_DAT20.BIN",     path="/CDROM/250_FEFE/",     lba=0,   size=0 },
			{ name="IDAT_A3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_A4.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_B2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_D.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_D2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_D3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IDAT_J.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_A3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_A4.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_B2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_D.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_D2.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_D3.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="IPRG_J.BIN",     path="/CDROM/260_IKKI/",     lba=0,   size=0 },
			{ name="BLOAD.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="BOOT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="BOOTDAT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ENDING.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ENDING1.SDD",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ENDING2.SDD",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT4.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT5.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="END_DAT6.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA4.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDA5.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="EVENTDAT.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="G1LOAD.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			--{ name="KNMLOGO.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="OVER.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="STAFF.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ST_END1.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ST_END2.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ST_END3.BIN",     path="/CDROM/270_BOOT/",     lba=0,   size=0 },
			{ name="ALL.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="ARMS.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="ARMSD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="ARMST.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="BOGU.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="CKD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="DISH.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="DOGU.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="EMBL.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="EX_ITEM.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="FNAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="FNO.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="FOOD.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="GAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="KEVENT.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="KOTEI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="MAGI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="MS.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="NAME.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="PARTY.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="SUKI.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="TABLE.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="TOWN.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="TRADE.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="VARIA.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="WAR_DAT.BIN",     path="/CDROM/280_GAME/",     lba=0,   size=0 },
			{ name="SD_WAR.BAK",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="SD_WAR.VAB",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_01.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_02.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_03.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_04.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_05.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_06.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_07.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_08.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_09.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_10.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_11.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_12.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_13.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_14.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_15.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="WAR_16.BIN",     path="/CDROM/290_WAR/",     lba=0,   size=0 },
			{ name="BOSS0.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="BOSS1.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="BOSS2.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO1.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO2.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO3.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO5.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="DEMO6.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="KONA5OLD.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="KONAMIC5.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="LOOP.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="OP0731.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="ROCK.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="_KONAMIC.STR",     path="/CDROM/300_MOVI/",     lba=0,   size=0 },
			{ name="YYY.STR",     path="/",     lba=0,   size=0 },
			{ name="ZZZ.STR",     path="/",     lba=0,   size=0 }
		  } --end filelist
		} -- end Game Japan
	} -- end games
} -- end PackageInfo
--------------------------------------------------------------------------------
--  GetCurrDirName                                                            --
--                                                                            --
--Returns the name of the current directory, given a full path.               --
--Expects a path minus the file name, e.g. /CDROM/010_ARA/                    --
--Returns the lowest directory represented (/010_ARA/).                       --
--                                                                            --
--Arguments:                                                                  --
--  path - A string representing the current absolute path.                   --
--                                                                            --
--------------------------------------------------------------------------------
function GetCurrDirName (path)
	local str, i, j;
	i, j = string.find(path, "/[^/]+/$");
	if(i ~= nil) then
		str = string.sub(path, i, j);
	end
	return str or "/";
end


--------------------------------------------------------------------------------
--  IsXA/IsSTR                                                                --
--                                                                            --
--Accepts a filename (just name or full path) and returns true if the         --
--extension is XA or STR.  Konami sometimes mangled or omitted certain        --
--properties when they built the disc, so examining the file system isn't     --
--very reliable.                                                              --
--                                                                            --
--Mainly required for determining mode when copying files (XA/STR are odd)    --
--                                                                            --
--Arguments:                                                                  --
--  path - A string ending in a file name.                                    --
--                                                                            --
--------------------------------------------------------------------------------
function IsXA (path)
	return string.upper(string.sub(path, string.len(path) - 1)) == "XA"
end

function IsSTR (path)
	return string.upper(string.sub(path, string.len(path) - 2)) == "STR"
end


function verify(cdutil, mainexe, workdir)
	local hFile, out;
	local direntELF = cdutil:findpath("/" .. mainexe .. ";1")
	if (direntELF == nil) then
		print ("Main exe not found")
		return false
	end
	local deFile = cdutil:findpath("/SYSTEM.CNF;1")
	if (deFile == nil) then
		print ("system.cnf not found")
		return false
	end

	print("Disc verified.")

	hFile = cdfile(deFile);
	out = Output(workdir .. "\\SYSTEM.CNF");
	out:copyfrom(hFile);
	out:close();
	hFile:close();
	return true
end

--------------------------------------------------------------------------------
--  GetDir                                                                    --
--                                                                            --
--Function to get a DirEnt object required for writing files to their proper  --
--location on the target image.                                               --
--                                                                            --
--CD-Tool is a little light on directory parsing, so the pathtree global      --
--exists to track what directories have been created, and to hold copies of   --
--their DirEnts for later use.  This function checks to see if a directory    --
--has already been created, and returns its DirEnt if it has.                 --
--                                                                            --
--If the requested directory has not yet been created, the function will do so--
--including creating any parent directories.  It then saves the created       --
--DirEnts, and returns the DirEnt of the lowest directory passed.             --
--                                                                            --
--Example:  /CDROM/010_ARA/ is passed.  If both exist, the DirEnt for 010_ARA --
--will be retrieved from pathtree and returned.  If 010_ARA does not exist, it--
--will be created, its DirEnt will be retained in pathtree, and then returned.--
--If neither exists, CDROM will be created and retained, followed by 010_ARA. --
--Again, 010_ARA's DirEnt will be returned.                                   --
--                                                                            --
--Arguments:                                                                  --
--  path - A string representing the current absolute path.                   --
--                                                                            --
--------------------------------------------------------------------------------
function GetDir(iso, path)
	local dirCurr;
	if(pathtree[path] == nil) then
		local strCurr   = GetCurrDirName(path);
		local strParent = string.gsub(path, strCurr, "") .. "/";
--  might have to create an additional parent for 150/170/180
		if(pathtree[strParent] == nil) then
			GetDir(iso, strParent);
		end
		local dirParent = pathtree[strParent].dir;
		local tmpd		 = cdutil:findpath(path);
		local nsect		 = tmpd.Size / 2048;
		dirCurr   = iso:createdir(dirParent, string.gsub(strCurr, "/", ""), nsect);
		if(tmpd:hasxa()) then dirCurr:setbasicsxa(); end
		pathtree[path] = { dir=dirCurr, parent=dirParent };
	else
		dirCurr = pathtree[path].dir;
	end
	return dirCurr;
end


--------------------------------------------------------------------------------
--  ApplyLBAPatch                                                             --
--                                                                            --
--Builds an array of Logical Block Addresses and file sizes (bytes) and writes--
--it to several files.  GSII does not seek using the table of contents  It    --
--uses this LBA listing to seek sectors directly.                             --
--                                                                            --
--Arguments:                                                                  --
--  None                                                                      --
--                                                                            --
--------------------------------------------------------------------------------
function ApplyLBAPatch (iso, workdir)
	local dirFile;
	local tgtFile;
	for i,v in ipairs(LBAPatchList) do
		print("Applying LBA Patch to " .. v.name)
		strPatchStatus = "Applying LBA Patch to " .. v.name
		if(iup) then coroutine.yield() end
		tgtFile = Input(workdir .. "\\" .. v.name);
		local buf = Buffer(true);
		local lba = GetLBAFromFileList(v.name, v.path);
		buf:copyfrom(tgtFile);
		buf:seek(v.seekpos);
		local tmp = buf:readU32();
		if(tmp ~= 68 and tmp ~= 0x9F800) then
			print(v.path .. v.name .. "Unable to apply LBA Patch.  Incorrect file position or table already modified");
		end
		buf:wseek(v.seekpos);
		for j,w in ipairs(FileList) do
			buf:writeU32(w.size);
			buf:writeU32(w.lba);
		end
		buf:seek(0);
		iso:putfile(buf, MODE2_FORM1, lba);
	end
end


--------------------------------------------------------------------------------
--  GetLBAFromFileList                                                        --
--                                                                            --
--The FileList saves the LBA of files written to the target ISO.  This        --
--function retrieves it, given a path and a file name.                        --
--                                                                            --
--Arguments:                                                                  --
--  name - name of the file                                                   --
--  path - path of the file, not including the file name.                     --
--------------------------------------------------------------------------------
function GetLBAFromFileList (name, path)
	local lba
	for i, v in ipairs(FileList) do
		if(v.name == name and v.path == path) then
			lba = v.lba;
			break;
		end
	end
	return lba;
end

--------------------------------------------------------------------------------
-- patches.lua                                                                --
-- 2012-05-09  - Pyriel                                                       --
--                                                                            --
--This script contains tables of patch data, and functions used to apply fixes--
--that are too simple/extensive to do reasonably through an ASM or BIN patch. --
--                                                                            --
--------------------------------------------------------------------------------

patch_gozz_data = {
	["GOZU"] = 		{ 0x41, 0x1F, 0x2A, 0x25, 0x00 },
	["MINOTAUR"] =	{ 0x47, 0x19, 0x1E, 0x1F, 0x24, 0x11, 0x25, 0x22, 0x00 }
}


circlet_files = {
	"VA05.BIN", "VA06.BIN", "VA08.BIN", "VA24.BIN", "VB04.BIN", "VB07.BIN",
	"VB09.BIN", "VB10.BIN", "VB12.BIN", "VC04.BIN", "VC07.BIN", "VC09.BIN",
	"VC28.BIN", "VC30.BIN", "VD07.BIN", "VE06.BIN", "VF01.BIN", "VF02.BIN",
	"VF04.BIN", "VI05.BIN", "VI10.BIN", "VI11.BIN", "VJ24.BIN", "VK06.BIN",
	"DOUGUYA1.BIN", "DOUGUYA2.BIN", "KAJIYA1.BIN", "KAJIYA2.BIN", "KANTEIY1.BIN",
	"KANTEIY2.BIN", "KOUEKI1.BIN", "KOUEKI2.BIN", "KPARTY1.BIN", "KPARTY2.BIN",
	"MONSYOY1.BIN", "MONSYOY2.BIN", "PARTYIN1.BIN", "PARTYIN2.BIN", "BOOK.BIN",
	"FUDAZUK.BIN", "HDOUGUYA.BIN", "HKAJIYA.BIN", "HMONSYOY.BIN", "KIKORI.BIN",
	"MOGURA.BIN", "PARTYCE1.BIN", "PARTYCH2.BIN", "PARTYCHG.BIN", "RBATTLE.BIN",
	"REST.BIN", "SOUKO.BIN", "STONE.BIN", "SYUGOITM.BIN", "VB19PIN.BIN",
	"VG08PIN.BIN", "BP0_AFT.BIN", "BOOT.BIN", "G1LOAD.BIN", "OVER.BIN", "BOGU.BIN"
}

--60 out of 1,100 files require the godspeed patch.
godspeed_files = {
	"VA05.BIN", "VA06.BIN", "VA08.BIN", "VA24.BIN", "VB04.BIN", "VB07.BIN",
	"VB09.BIN", "VB10.BIN", "VB12.BIN", "VC04.BIN", "VC07.BIN", "VC09.BIN",
	"VC28.BIN", "VC30.BIN", "VD07.BIN", "VE06.BIN", "VF01.BIN", "VF02.BIN",
	"VF04.BIN", "VI05.BIN", "VI10.BIN", "VI11.BIN", "VJ24.BIN", "VK06.BIN",
	"DOUGUYA1.BIN", "DOUGUYA2.BIN", "KAJIYA1.BIN", "KAJIYA2.BIN", "KANTEIY1.BIN",
	"KANTEIY2.BIN", "KOUEKI1.BIN", "KOUEKI2.BIN", "KPARTY1.BIN", "KPARTY2.BIN",
	"MONSYOY1.BIN", "MONSYOY2.BIN", "PARTYIN1.BIN", "PARTYIN2.BIN", "FUDAZUK.BIN",
	"HDOUGUYA.BIN", "HKAJIYA.BIN", "HMONSYOY.BIN", "KIKORI.BIN", "MOGURA.BIN",
	"PARTYCE1.BIN", "PARTYCH2.BIN", "PARTYCHG.BIN", "RBATTLE.BIN", "REST.BIN",
	"SOUKO.BIN", "STONE.BIN", "SYUGOITM.BIN", "TANTEI.BIN","VB19PIN.BIN",
	"VG08PIN.BIN", "BP0_AFT.BIN", "BOOT.BIN", "G1LOAD.BIN", "OVER.BIN", "EMBL.BIN"
}

--61 out of 1,100 files require the gozz patch.
gozz_files = {
	"VA05.BIN", "VA06.BIN", "VA08.BIN", "VA24.BIN", "VB04.BIN", "VB07.BIN",
	"VB09.BIN", "VB10.BIN", "VB12.BIN", "VC04.BIN", "VC07.BIN", "VC09.BIN",
	"VC28.BIN", "VC30.BIN", "VD07.BIN", "VE06.BIN", "VF01.BIN", "VF02.BIN",
	"VF04.BIN", "VG11.BIN", "VI05.BIN", "VI10.BIN", "VI11.BIN", "VJ24.BIN",
	"VK06.BIN",	"DOUGUYA1.BIN", "DOUGUYA2.BIN", "KAJIYA1.BIN", "KAJIYA2.BIN", "KANTEIY1.BIN",
	"KANTEIY2.BIN", "KOUEKI1.BIN", "KOUEKI2.BIN", "KPARTY1.BIN", "KPARTY2.BIN",
	"MONSYOY1.BIN", "MONSYOY2.BIN", "PARTYIN1.BIN", "PARTYIN2.BIN", "FUDAZUK.BIN",
	"HDOUGUYA.BIN", "HKAJIYA.BIN", "HMONSYOY.BIN", "KIKORI.BIN", "MOGURA.BIN",
	"PARTYCE1.BIN", "PARTYCH2.BIN", "PARTYCHG.BIN", "RBATTLE.BIN", "REST.BIN",
	"SOUKO.BIN", "STONE.BIN", "SYUGOITM.BIN", "VB19PIN.BIN", "VG08PIN.BIN",
	"BP0_AFT.BIN", "BOOT.BIN", "G1LOAD.BIN", "OVER.BIN", "EMBL.BIN", "MAGI.BIN",
}

function PatchRequested(name)
	local i, v;
	for i, v in ipairs(patchreq) do
		if(v.file == name) then
			return true;
		end
	end
	return false;
end

function ApplyPatch(patch, file)
	local i, j, v, patchbyte;
	for j, v in pairs(patch) do
		for i, patchbyte in ipairs(v) do
			file[j + i - 1] = patchbyte;
		end
	end
end

function ApplyBinFilePatches(name, deFile)
	local i, v;
	local touched = false;
	local file = Buffer(true);
	for i, v in ipairs(patchreq) do
		if(v.file == name) then
			if(v.patch == nil) then
				if(touched == true) then error("Cannot replace file from HDD after another patch has been applied.  Check patch order.") end
				print("PATCHER: Loading " .. name .. " from pre-patched file on HDD for " .. v.name);
				local tmp = Input(name);
				file:copyfrom(tmp);
				touched = true;
			else
				if(touched == false) then
					print("PATCHER:  Loading " .. name .. " from source disc.");
					local tmp = cdfile(deFile);
					file:copyfrom(tmp);
					touched = true;
				end
				print("PATCHER: Updating " .. name .. " with " .. v.name);
				ApplyPatch(v.patch, file);
			end
		end
	end
	if(touched == false) then
		print("No patches found for file " .. name .. " copying from source disc.");
		file = cdfile(deFile);
	end
	return file;
end

function IsGodspeedFile(name)
	for i, v in ipairs(godspeed_files) do
		if (v==name) then
			return true
		end
	end
	return false;
end

function ApplyGodspeedPatch(file, name)
	local i, touched = false;
	i = 0
	while (i < file:getsize()) do
		if(file[i] == 0x4E
		 and file[i + 1] == 0x22
		 and file[i + 2] == 0x25
		 and file[i + 3] == 0x15
		 and file[i + 4] == 0x10
		 and file[i + 5] == 0x42
		 and file[i + 6] == 0x1F
		 and file[i + 7] == 0x1C
		 and file[i + 8] == 0x29) then
			file:wseek(i)
			file:writeU32(0x23141F41);
			file:writeU32(0x14151520);
			--Rune name appears in an investigation, so may need to preserve the remaining string.
			local j = i + 9
			while (file[j] ~= 0) do
				file:writeU8(file[j]);
				j=j+1;
			end
			file:writeU8(0);
			i = file:seek(file:wtell());
			touched = true;
		end
		i=i+1;
	end
	file:seek(file:wseek(0));
	if(touched == true) then print("PATCHER: Applied Godspeed Patch to "..name); end
end

function IsGozzFile(name)
	for i, v in ipairs(gozz_files) do
		if (v==name) then
			return true
		end
	end
	return false;
end

function ApplyGozzPatch(file, name, gozztype)
	local i, touched = false;
	i = 0
	while (i < file:getsize()) do
		if(file[i] == 0x41
		 and file[i + 1] == 0x1F
		 and file[i + 2] == 0x2A
		 and file[i + 3] == 0x2A
		 and file[i + 4] == 0x00) then
			local tmp_patch = { [i] = patch_gozz_data[gozztype] };
			ApplyPatch(tmp_patch, file);
			i = i + 5;
			touched = true;
		end
		i=i+1;
	end
	file:seek(file:wseek(0));
	if(touched == true) then print("PATCHER: Applied Gozz Rune Patch to "..name); end
end

function IsCircletFile(name)
	for i, v in ipairs(circlet_files) do
		if (v==name) then
			return true
		end
	end
	return false;
end

function ApplyCircletPatch(file, name)
	local i, touched = false;
	i = 0
	while (i < file:getsize()) do
		if(file[i] == 0x3D
		 and file[i + 1] == 0x19
		 and file[i + 2] == 0x22
		 and file[i + 3] == 0x13
		 and file[i + 4] == 0x25
		 and file[i + 5] == 0x22
		 and file[i + 6] == 0x15
		 and file[i + 7] == 0x24) then
			file:wseek(i)
			file:writeU32(0x1322193D);
			file:writeU8(0x1C);
			file:writeU8(0x15);
			file:writeU8(0x24);
			--Item name appears in a book, so may need to preserve the remaining string.
			local j = i + 8
			while (file[j] ~= 0) do
				file:writeU8(file[j]);
				j=j+1;
			end
			file:writeU8(0);
			i = file:seek(file:wtell());
			touched = true;
		end
		i=i+1;
	end
	file:seek(file:wseek(0));
	if(touched == true) then print("PATCHER: Applied Circlet Patch to "..name); end
end

function IsSearchPatchFile(name)
	return (IsCircletFile(name) or IsGodspeedFile(name) or IsGozzFile(name));
end

function extract(cdutil, workdir)
	for i,v in ipairs(FileList) do
		strPatchStatus = "Extracting file " .. v.name
		coroutine.yield()
		local isopath = v.path .. v.name .. ";1"
		local deFile = cdutil:findpath(isopath)
		local hFile
		local tree
		local out
		if (IsXA(v.name) or (IsSTR(v.name) and v.name ~= "ZZZ.STR")) then
--  Copy XA/STR files by sectors.  Copying without explicitly setting the secord and size (even MODE_RAW) hoses the data.
--  Note that ZZZ.STR is not a STR file, or at least not a valid one.
			hFile = cdfile(deFile["Sector"], (deFile["Size"] / 2048) * 2336, MODE2)
		else
			hFile = cdfile(deFile)
		end
		out = Output(workdir .. "\\" .. v.name)
		out:copyfrom(hFile)
		out:close()
		hFile:close()
		intPatchProgress = intPatchProgress + 1
	end
end

function buildcd(cdutil, iso, workdir)
	local direntELF, targetRoot;
	local hCurrFile, pvd, exefile;

	intPatchProgress = intPatchProgress + 10
	strPatchStatus = "Initializing output image..."
	coroutine.yield()
	iup.LoopStep()
	--  Create the first 16 sectors of the target ISO, i.e., the license data, from the source.
	iso:foreword(cdutil);
	--  Create the Primary Volume Descriptor (PVD) from the source.
	pvd = createpvd(cdutil);
	targetRoot = iso:setbasics(pvd);

--  Start out the pathtree with the entry for the root directory.
--  Other directories will be created on the ISO and added to the pathtree when
--  they are first encountered in the file list.
	pathtree["/"] = { dir=targetRoot, parent=targetRoot };
	for i,v in ipairs(FileList) do
		local isopath = v.path .. v.name .. ";1";
		local deFile = cdutil:findpath(isopath) or error("Could not find file: " .. isopath .. " on source image/cd");
		local hFile;
		local tree;
		local out;

		strPatchStatus = "Adding " .. v.name .. " to target and applying text fixes..."
		coroutine.yield()
		GetDir(iso, v.path);
		hFile = Input(workdir .. "\\" .. v.name);

		if (IsXA(v.name) or (IsSTR(v.name) and v.name ~= "ZZZ.STR")) then
--  Copy XA/STR files by sectors.  Copying without explicitly setting the secord and size (even MODE_RAW) hoses the data.
--  Note that ZZZ.STR is not a STR file, or at least not a valid one.
			if (v.name == "BGM.XA") then
			--Konami seems to have relied on the interleave of BGM.XA starting at sector 344, about 6 seconds and a few blocks into the disc
			--It didn't work out so well when they built the disc with a PSX EXE ~2KB smaller than in Japan for the NA region.
				local dummysector = {}
				for i = iso:getdispsect(), 344, 1 do
				--  iso:createsector(dummysector, MODE2)
				i = 344
				end
			end
			v.lba = iso:getdispsect();
			v.size = (hFile:getsize() / 2048) * 2336;
			tree = iso:createfile(pathtree[v.path].dir, v.name, hFile, deFile, MODE2)
			tree.size = deFile.Size
		else
			if(IsSearchPatchFile(v.name)) then
				local tmp = Buffer(true);
				tmp:copyfrom(hFile);
				hFile:close();
				if(SearchPatches["godspeed"].Active == 1 and IsGodspeedFile(v.name)) then
					ApplyGodspeedPatch(tmp, v.name);
				end
				if(SearchPatches["gozz"].Active == 1 and IsGozzFile(v.name)) then
					ApplyGozzPatch(tmp, v.name, SearchPatches["gozz"].Type);
				end
				if(SearchPatches["circlet"].Active == 1 and IsCircletFile(v.name)) then
					ApplyCircletPatch(tmp, v.name);
				end
				local out = Output(workdir .. "\\" .. v.name);
				out:copyfrom(tmp);
				out:close();
				tmp:close();
				hFile = Input(workdir .. "\\" .. v.name);
			end
			v.lba = iso:getdispsect();
			v.size = hFile:getsize();
			tree = iso:createfile(pathtree[v.path].dir, v.name, hFile, deFile);
			tree.size = deFile.Size
		end
		hFile:close();
		intPatchProgress = intPatchProgress + 1
	end

	--  Pad out the ISO, if possible.
	strPatchStatus = "Padding ISO (if necessary)..."
	coroutine.yield()
	local dummysector = {}
    	for i = iso:getdispsect(), finalsector, 1 do
    	    iso:createsector(dummysector, MODE2)
 	end
	intPatchProgress = intPatchProgress + 10

	strPatchStatus = "Applying LBA Patch to required files..."
	coroutine.yield()
	ApplyLBAPatch(iso, workdir);
	intPatchProgress = intPatchProgress + 50
	coroutine.yield()
end

function ApplyFilePatch(files, workdir)
	local ret
	for i, file in ipairs(files) do
		if (file.Type == "PPF") then
			strPatchStatus = "Patching " .. file.GameFileName .. " from PPF file " .. file.PatchFileName
			coroutine.yield()
			ret = PpfPatchFile(workdir .. "\\" .. file.GameFileName, patchdir .. "\\" .. file.PatchFileName)
			if(ret ~= 0) then
				print("ERROR: Could not apply patch file " .. file.PatchFileName .. " returned code " .. tostring(ret))
				return ret
			end
		end
	end
	return 0
end

function ApplySearchPatch(searches)
	for i, search in ipairs(searches) do
		SearchPatches[search.VarQual].Active = 1
		SearchPatches[search.VarQual].Type = search.Replace
		print("SEARCH: Activated " .. search.VarQual .. " with value " .. search.Replace)
	end
end

function ApplyPatches (patches, workdir, patchdir)
	local ret = 0
	for i, patch in ipairs(patches) do
		if (patch.Active == 1) then
			if(patch.Files ~= nil) then
				ret = ApplyFilePatch(patch.Files, workdir, patchdir)
			end
			if(patch.Search ~= nil) then
				ApplySearchPatch(patch.Search)
			end
			intPatchProgress = intPatchProgress + 20
			if(ret ~= 0) then
				print("ERROR: Failed applying patches.")
				return ret
			end
		end
	end
	return 0
end

function GetMainExeName (GameId)
	if(PackageInfo.Games[GameId] ~= nil) then
		return PackageInfo.Games[GameId].Serial
	end
	return nil
end

function GetRegionInfo (GameId)
	if(PackageInfo.Games[GameId] ~= nil) then
		return PackageInfo.Games[GameId].Region .. " (" .. PackageInfo.Games[GameId].Language .. ")"
	end
	return nil
end

function GetPatchDir (GameId)
	if(PackageInfo.Games[GameId] ~= nil) then
		return PackageInfo.Games[GameId].PatchDir
	end
	return nil
end

function GetFileList (GameId)
	if(PackageInfo.Games[GameId] ~= nil) then
		return PackageInfo.Games[GameId].FileList
	end
	return nil
end

function GetLBAList (GameId)
	print("Get LBA List")
	if(PackageInfo.Games[GameId] ~= nil) then
		print("LBA - " .. type(PackageInfo.Games[GameId].LBAPatchList) .. " " .. #PackageInfo.Games[GameId].LBAPatchList)
		return PackageInfo.Games[GameId].LBAPatchList
	end
	return nil
end

function GetPatchIdx (GameId, PatchName)
	for i, v in ipairs(PackageInfo.Games[GameId].Patches) do
		if(v.Name == PatchName) then
			return i
		end
	end
	return nil
end

function GetPatch (GameId, PatchName)
	for i, v in ipairs(PackageInfo.Games[GameId].Patches) do
		if(v.Name == PatchName) then
			return v
		end
	end
	return nil
end

function GetPatches (GameId)
	return PackageInfo.Games[GameId].Patches
end


--Toggle=1 indicates the patch is highlighted in the list that owns it.
--IUP is sort of bizarre in that you can't seem to retrieve item statuses
--except in the callbacks, or maybe I just can't find it in the cruddy docs.
function TogglePatch(GameId, SenderId, PatchName, State)
	local intPatchIdx = GetPatchIdx(GameId, PatchName)
	if(intPatchIdx == nil) then
		iup.Message("Patch not found", "Unable to find the patch " .. PatchName .. " in the package.")
	else
		PackageInfo.Games[GameId].Patches[intPatchIdx].Toggle = SenderId
	end
end

function ClearToggle(GameId)
	for i, v in ipairs(PackageInfo.Games[GameId].Patches) do
		v.Toggle = 0
	end
end

function UpdatePatchStatuses(GameId, SenderId)
	for i, v in ipairs(PackageInfo.Games[GameId].Patches) do
		if(v.Toggle == SenderId) then
			v.Toggle = 0
			v.Active = bit.bxor(v.Active, 1)
			if(v.Active == 1 and v.Excludes ~= nil) then
				for i, v in ipairs(v.Excludes) do
					DeactivatePatch(GameId, v)
				end
			end
		end
	end
end

function SetPatchState (GameId, PatchName, State)
	local intPatchIdx = GetPatchIdx(GameId, PatchName)
	if(intPatchIdx == nil) then
		iup.Message("Patch not found", "Unable to find the patch " .. PatchName .. " in the package.")
	else
		PackageInfo.Games[GameId].Patches[intPatchIdx].Active = State
		PackageInfo.Games[GameId].Patches[intPatchIdx].Toggle = 0
	end
end

function HasActivePatches(GameId)
	for i, v in ipairs(PackageInfo.Games[GameId].Patches) do
		if (v.Active == 1) then
			return true
		end
	end
	return false
end

function ActivatePatch(GameId, PatchName)
	SetPatchState(GameId, PatchName, 1)
end

function DeactivatePatch(GameId, PatchName)
	SetPatchState(GameId, PatchName, 0)
end

-------------------------------------------------------------------------------
--ppf.lua - Custom PPF file implementation.                                  --
--                                                                           --
--The patch requires a format that can handle files of around 150 MB, but it --
--also needs to be able to extend the files, which PPF tools won't do.       --
--Rolling up a custom implementation that can add data to files, in addition --
--to replacing it.                                                           --
--                                                                           --
--Format:	File ID - 5 bytes - "PPF40", magic value                         --
--			Encoding Version - 1 byte - 0xFF								 --
--			Patch Description - 50 bytes - Text describing the patch         --
--          Image Type - 1 byte - Always 0 for binary file                   --
--          Validation Flag - 1 byte - Always 0, validation not supported    --
--          Undo Flag - 1 byte - Always 0, undo not supported                --
--          Expansion - 1 byte - Unused value                                --
--          Patch Instructions:                                              --
--                  Command - 1 byte - Value 0 for "replace", 1 for "add"    --
--                  Offset - 4 bytes - Offset into the file.  0 for adds     --
--                  Count - 1 byte - Number of bytes to write                --
--                  Patch Data - (Count) bytes - bytes to write into the file--
-------------------------------------------------------------------------------

loadmodule "lualibs"
loadmodule "luahandle"

S_FILEID		= "PPF40"
N_FILEID_LENGTH = 5
N_FILEDESC_LENGTH = 50
ERROR_BAD_FILEID = -1
ERROR_BAD_VERSION = -2
ERROR_BAD_FLAGS = -3
ERROR_NODATA = -4
ERROR_BAD_REPLACE = -5
ERROR_BAD_OFFSET = -6
ERROR_BAD_SIZE = -7
ERROR_BAD_PATCH_INST = -8

function PpfApplyPatch (TargetFile, PatchFile)
--I had forgotten about the size property, and thought it was missing from file handles
	local FileId = "", tmp, PatchFileEnd, TargetFileEnd, mode
	PatchFile:seek(0, SEEK_END)
	PatchFileEnd = PatchFile:tell()
	PatchFile:seek(0, SEEK_SET)
	TargetFile:seek(0, SEEK_END)
	TargetFileEnd = TargetFile:tell()
	TargetFile:seek(0, SEEK_SET)

	if(PatchFileEnd - PatchFile:tell() < 0x43 or TargetFileEnd - TargetFile:tell() < 1) then return ERROR_NODATA end

	for i = 1, N_FILEID_LENGTH do
		FileId = FileId .. string.char(PatchFile:readU8())
	end
	if(FileId ~= S_FILEID) then
		return ERROR_BAD_FILEID
	end

	tmp = PatchFile:readU8()
	if(tmp ~= 0xFF) then
		return ERROR_BAD_VERSION
	end

	for i = 1, N_FILEDESC_LENGTH do
		PatchFile:readU8()
	end
	tmp = PatchFile:readU32()
	if(tmp ~= 0) then
		return ERROR_BAD_FLAGS
	end

	mode = "REPLACE"
	local bytecount = 0
	while (PatchFile:tell() < PatchFileEnd) do
		if(PatchFile:tell() + 7 > PatchFileEnd) then return ERROR_BAD_PATCH_INST end
		local cmd = PatchFile:readU8()
		local offset = PatchFile:readU32()
		local size = PatchFile:readU8()
		if(cmd == 0) then
			if(mode == "ADD") then return ERROR_BAD_REPLACE end
			if(TargetFileEnd < offset) then return ERROR_BAD_OFFSET end
			if(PatchFile:tell() + size > PatchFileEnd or offset + size > TargetFileEnd) then return ERROR_BAD_SIZE end
			TargetFile:seek(offset)
			for i = 1, size do
				tmp = PatchFile:readU8()
				TargetFile:writeU8(tmp)
				bytecount = bytecount + 1
			end
		elseif(cmd == 1) then
			if(mode == "REPLACE") then
				mode = "ADD"
				TargetFile:seek(TargetFileEnd)
			end
			if(PatchFile:tell() + size > PatchFileEnd) then return ERROR_BAD_SIZE end
			for i = 1, size do
				tmp = PatchFile:readU8()
				TargetFile:writeU8(tmp)
				bytecount = bytecount + 1
			end
		end

		--keep responsive
		if(iup and bytecount > 100000) then
			bytecount = 0
			coroutine.yield()
		end
	end
	return 0
end

function PpfPatchFile (TargetFileName, PatchFileName)
	local TargetFile = Input(TargetFileName)
	local buff = Buffer(true)
	buff:copyfrom(TargetFile)
	if(iup) then coroutine.yield() end
	TargetFile:close()
	TargetFile = Output(TargetFileName)
	TargetFile:copyfrom(buff)

	local PatchFile = Input(PatchFileName)

	local ret = PpfApplyPatch(TargetFile, PatchFile)
	--if(ret ~= 0) then print("Error:" .. ret) end
	TargetFile:flush()
	TargetFile:close()
	PatchFile:close()
	return ret
end
